searchData={"items":[{"type":"module","doc":"High-level DSL for defining MCP (Model Context Protocol) clients.\n\nThis module provides an Ecto-like interface for creating MCP clients with minimal boilerplate.\nBy using this module, you get a fully functional MCP client with automatic supervision,\ntransport management, and all standard MCP operations.","title":"Hermes.Client","ref":"Hermes.Client.html"},{"type":"module","doc":"Define a client module:\n\n    defmodule MyApp.AnthropicClient do\n      use Hermes.Client,\n        name: \"MyApp\",\n        version: \"1.0.0\",\n        protocol_version: \"2024-11-05\",\n        capabilities: [:roots, {:sampling, list_changed?: true}]\n    end\n\nAdd it to your supervision tree:\n\n    children = [\n      {MyApp.AnthropicClient, \n       transport: {:stdio, command: \"uvx\", args: [\"mcp-server-anthropic\"]}}\n    ]\n\nUse the client:\n\n    {:ok, tools} = MyApp.AnthropicClient.list_tools()\n    {:ok, result} = MyApp.AnthropicClient.call_tool(\"search\", %{query: \"elixir\"})","title":"Usage - Hermes.Client","ref":"Hermes.Client.html#module-usage"},{"type":"module","doc":"The `use` macro accepts the following required options:\n\n  * `:name` - The client name to advertise to the server (string)\n  * `:version` - The client version (string)\n  * `:protocol_version` - The MCP protocol version (string)\n  * `:capabilities` - List of client capabilities (see below)","title":"Options - Hermes.Client","ref":"Hermes.Client.html#module-options"},{"type":"module","doc":"Capabilities can be specified as:\n\n  * Atoms: `:roots`, `:sampling`\n  * Tuples with options: `{:roots, list_changed?: true}`\n  * Maps for custom capabilities: `%{\"custom\" => %{\"feature\" => true}}`","title":"Capabilities - Hermes.Client","ref":"Hermes.Client.html#module-capabilities"},{"type":"module","doc":"When starting the client, you must provide transport configuration:\n\n  * `{:stdio, command: \"cmd\", args: [\"arg1\", \"arg2\"]}`\n  * `{:sse, base_url: \"http://localhost:8000\"}`\n  * `{:websocket, url: \"ws://localhost:8000/ws\"}`\n  * `{:streamable_http, url: \"http://localhost:8000/mcp\"}`","title":"Transport Configuration - Hermes.Client","ref":"Hermes.Client.html#module-transport-configuration"},{"type":"module","doc":"By default, the client process is registered with the module name.\nYou can override this with the `:name` option in `child_spec` or `start_link`:\n\n    # Custom atom name\n    {MyApp.AnthropicClient, name: :my_custom_client, transport: ...}\n    \n    # For distributed systems with registries (e.g., Horde)\n    {MyApp.AnthropicClient,\n     name: {:via, Horde.Registry, {MyCluster, \"client_1\"}},\n     transport_name: {:via, Horde.Registry, {MyCluster, \"transport_1\"}},\n     transport: ...}\n\nWhen using via tuples or other non-atom names, you must explicitly provide\nthe `:transport_name` option. For atom names, the transport is automatically\nnamed as `Module.concat(ClientName, \"Transport\")`.","title":"Process Naming - Hermes.Client","ref":"Hermes.Client.html#module-process-naming"},{"type":"macro","doc":"Generates an MCP client module with all necessary functions.\n\nThis macro is used via the `use` directive and accepts the following options:\n\n  * `:name` - Client name (required, string)\n  * `:version` - Client version (required, string)  \n  * `:protocol_version` - MCP protocol version (required, string)\n  * `:capabilities` - List of capabilities (optional, defaults to empty list)\n\nThe macro generates:\n\n  * `child_spec/1` - For supervision tree integration\n  * `start_link/1` - To start the client\n  * All MCP operation functions (ping, list_tools, call_tool, etc.)","title":"Hermes.Client.__using__/1","ref":"Hermes.Client.html#__using__/1"},{"type":"type","doc":"","title":"Hermes.Client.capabilities/0","ref":"Hermes.Client.html#t:capabilities/0"},{"type":"type","doc":"","title":"Hermes.Client.capability/0","ref":"Hermes.Client.html#t:capability/0"},{"type":"type","doc":"","title":"Hermes.Client.capability_opts/0","ref":"Hermes.Client.html#t:capability_opts/0"},{"type":"macro","doc":"Guard to check if an atom is a valid client capability.","title":"Hermes.Client.is_client_capability/1","ref":"Hermes.Client.html#is_client_capability/1"},{"type":"macro","doc":"Guard to check if a capability is supported by checking map keys.","title":"Hermes.Client.is_supported_capability/2","ref":"Hermes.Client.html#is_supported_capability/2"},{"type":"module","doc":"Represents an operation to be performed by the MCP client.\n\nThis struct encapsulates all information about a client API call:\n- `method` - The MCP method to call\n- `params` - The parameters to send to the server\n- `progress_opts` - Progress tracking options (optional)\n- `timeout` - The timeout for this specific operation (default: 30 seconds)","title":"Hermes.Client.Operation","ref":"Hermes.Client.Operation.html"},{"type":"function","doc":"Creates a new operation struct.","title":"Hermes.Client.Operation.new/1","ref":"Hermes.Client.Operation.html#new/1"},{"type":"function","doc":"* `attrs` - Map containing the operation attributes\n    * `:method` - The MCP method name (required)\n    * `:params` - The parameters to send to the server (required)\n    * `:progress_opts` - Progress tracking options (optional)\n    * `:timeout` - The timeout for this operation in milliseconds (optional, defaults to 30s)","title":"Parameters - Hermes.Client.Operation.new/1","ref":"Hermes.Client.Operation.html#new/1-parameters"},{"type":"type","doc":"","title":"Hermes.Client.Operation.progress_options/0","ref":"Hermes.Client.Operation.html#t:progress_options/0"},{"type":"type","doc":"","title":"Hermes.Client.Operation.t/0","ref":"Hermes.Client.Operation.html#t:t/0"},{"type":"module","doc":"Fluent API for building MCP protocol errors.\n\nThis module provides a semantic interface for creating errors that comply with\nthe MCP specification and JSON-RPC 2.0 error standards.","title":"Hermes.MCP.Error","ref":"Hermes.MCP.Error.html"},{"type":"module","doc":"- **Protocol Errors**: Standard JSON-RPC errors (parse, invalid request, etc.)\n- **Transport Errors**: Connection and communication issues\n- **Resource Errors**: MCP-specific resource handling errors\n- **Execution Errors**: Tool and operation execution failures","title":"Error Categories - Hermes.MCP.Error","ref":"Hermes.MCP.Error.html#module-error-categories"},{"type":"module","doc":"# Protocol errors\n    Hermes.MCP.Error.protocol(:parse_error)\n    Hermes.MCP.Error.protocol(:method_not_found, %{method: \"unknown\"})\n\n    # Transport errors\n    Hermes.MCP.Error.transport(:connection_refused)\n    Hermes.MCP.Error.transport(:timeout, %{elapsed_ms: 30000})\n\n    # Resource errors\n    Hermes.MCP.Error.resource(:not_found, %{uri: \"file:///missing.txt\"})\n\n    # Execution errors with custom messages\n    Hermes.MCP.Error.execution(\"Database connection failed\", %{retries: 3})\n\n    # Converting from JSON-RPC\n    Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32700, \"message\" => \"Parse error\"})","title":"Examples - Hermes.MCP.Error","ref":"Hermes.MCP.Error.html#module-examples"},{"type":"function","doc":"","title":"Hermes.MCP.Error.build_json_rpc/2","ref":"Hermes.MCP.Error.html#build_json_rpc/2"},{"type":"function","doc":"Creates an execution error with a custom message.\n\nUsed for tool execution failures and domain-specific errors.","title":"Hermes.MCP.Error.execution/2","ref":"Hermes.MCP.Error.html#execution/2"},{"type":"function","doc":"iex> Hermes.MCP.Error.execution(\"Database connection failed\")\n    %Hermes.MCP.Error{code: -32000, reason: :execution_error, message: \"Database connection failed\", data: %{}}\n\n    iex> Hermes.MCP.Error.execution(\"API rate limit exceeded\", %{retry_after: 60})\n    %Hermes.MCP.Error{code: -32000, reason: :execution_error, message: \"API rate limit exceeded\", data: %{retry_after: 60}}","title":"Examples - Hermes.MCP.Error.execution/2","ref":"Hermes.MCP.Error.html#execution/2-examples"},{"type":"function","doc":"Creates an error from a JSON-RPC error object.","title":"Hermes.MCP.Error.from_json_rpc/1","ref":"Hermes.MCP.Error.html#from_json_rpc/1"},{"type":"function","doc":"iex> Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32700, \"message\" => \"Parse error\"})\n    %Hermes.MCP.Error{code: -32700, reason: :parse_error, message: \"Parse error\", data: %{}}\n\n    iex> Hermes.MCP.Error.from_json_rpc(%{\"code\" => -32002, \"message\" => \"Not found\", \"data\" => %{\"uri\" => \"/file\"}})\n    %Hermes.MCP.Error{code: -32002, reason: :resource_not_found, message: \"Not found\", data: %{\"uri\" => \"/file\"}}","title":"Examples - Hermes.MCP.Error.from_json_rpc/1","ref":"Hermes.MCP.Error.html#from_json_rpc/1-examples"},{"type":"function","doc":"Creates a protocol-level error.\n\nThese are standard JSON-RPC errors that occur during message parsing and validation.","title":"Hermes.MCP.Error.protocol/2","ref":"Hermes.MCP.Error.html#protocol/2"},{"type":"function","doc":"- `:parse_error` - Invalid JSON was received\n- `:invalid_request` - The JSON is not a valid Request object\n- `:method_not_found` - The method does not exist\n- `:invalid_params` - Invalid method parameters\n- `:internal_error` - Internal JSON-RPC error","title":"Supported Reasons - Hermes.MCP.Error.protocol/2","ref":"Hermes.MCP.Error.html#protocol/2-supported-reasons"},{"type":"function","doc":"iex> Hermes.MCP.Error.protocol(:parse_error)\n    %Hermes.MCP.Error{code: -32700, reason: :parse_error, message: \"Parse error\", data: %{}}\n\n    iex> Hermes.MCP.Error.protocol(:method_not_found, %{method: \"foo\"})\n    %Hermes.MCP.Error{code: -32601, reason: :method_not_found, message: \"Method not found\", data: %{method: \"foo\"}}","title":"Examples - Hermes.MCP.Error.protocol/2","ref":"Hermes.MCP.Error.html#protocol/2-examples"},{"type":"function","doc":"Creates a resource-specific error.\n\nUsed for MCP resource operations.","title":"Hermes.MCP.Error.resource/2","ref":"Hermes.MCP.Error.html#resource/2"},{"type":"function","doc":"iex> Hermes.MCP.Error.resource(:not_found, %{uri: \"file:///missing.txt\"})\n    %Hermes.MCP.Error{code: -32002, reason: :resource_not_found, message: \"Resource not found\", data: %{uri: \"file:///missing.txt\"}}","title":"Examples - Hermes.MCP.Error.resource/2","ref":"Hermes.MCP.Error.html#resource/2-examples"},{"type":"type","doc":"","title":"Hermes.MCP.Error.t/0","ref":"Hermes.MCP.Error.html#t:t/0"},{"type":"function","doc":"Encodes the error as a JSON-RPC error response.","title":"Hermes.MCP.Error.to_json_rpc/2","ref":"Hermes.MCP.Error.html#to_json_rpc/2"},{"type":"function","doc":"iex> error = Hermes.MCP.Error.protocol(:parse_error)\n    iex> {:ok, encoded} = Hermes.MCP.Error.to_json_rpc(error, \"req-123\")\n    iex> String.contains?(encoded, \"Parse error\")\n    true","title":"Examples - Hermes.MCP.Error.to_json_rpc/2","ref":"Hermes.MCP.Error.html#to_json_rpc/2-examples"},{"type":"function","doc":"Creates a transport-level error.\n\nUsed for network, connection, and communication failures.","title":"Hermes.MCP.Error.transport/2","ref":"Hermes.MCP.Error.html#transport/2"},{"type":"function","doc":"iex> Hermes.MCP.Error.transport(:connection_refused)\n    %Hermes.MCP.Error{code: -32000, reason: :connection_refused, message: \"Connection Refused\", data: %{}}\n\n    iex> Hermes.MCP.Error.transport(:timeout, %{elapsed_ms: 5000})\n    %Hermes.MCP.Error{code: -32000, reason: :timeout, message: \"Timeout\", data: %{elapsed_ms: 5000}}","title":"Examples - Hermes.MCP.Error.transport/2","ref":"Hermes.MCP.Error.html#transport/2-examples"},{"type":"module","doc":"Handles parsing and validation of MCP (Model Context Protocol) messages using the Peri library.\n\nThis module provides functions to parse and validate MCP messages based on the Model Context Protocol schema","title":"Hermes.MCP.Message","ref":"Hermes.MCP.Message.html"},{"type":"function","doc":"","title":"Hermes.MCP.Message.batch_schema/1","ref":"Hermes.MCP.Message.html#batch_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.batch_schema!/1","ref":"Hermes.MCP.Message.html#batch_schema!/1"},{"type":"function","doc":"Builds an error message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","title":"Hermes.MCP.Message.build_error/2","ref":"Hermes.MCP.Message.html#build_error/2"},{"type":"function","doc":"* `error` - The error map with code, message, and optional data\n  * `id` - The error response ID","title":"Parameters - Hermes.MCP.Message.build_error/2","ref":"Hermes.MCP.Message.html#build_error/2-parameters"},{"type":"function","doc":"iex> Message.build_error(%{\"code\" => -32600, \"message\" => \"Invalid Request\"}, \"req_123\")\n    %{\"jsonrpc\" => \"2.0\", \"error\" => %{\"code\" => -32600, \"message\" => \"Invalid Request\"}, \"id\" => \"req_123\"}","title":"Examples - Hermes.MCP.Message.build_error/2","ref":"Hermes.MCP.Message.html#build_error/2-examples"},{"type":"function","doc":"Builds a notification message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","title":"Hermes.MCP.Message.build_notification/2","ref":"Hermes.MCP.Message.html#build_notification/2"},{"type":"function","doc":"* `method` - The notification method\n  * `params` - The notification parameters","title":"Parameters - Hermes.MCP.Message.build_notification/2","ref":"Hermes.MCP.Message.html#build_notification/2-parameters"},{"type":"function","doc":"iex> Message.build_notification(\"notifications/message\", %{\"level\" => \"info\", \"data\" => \"test\"})\n    %{\"jsonrpc\" => \"2.0\", \"method\" => \"notifications/message\", \"params\" => %{\"level\" => \"info\", \"data\" => \"test\"}}","title":"Examples - Hermes.MCP.Message.build_notification/2","ref":"Hermes.MCP.Message.html#build_notification/2-examples"},{"type":"function","doc":"Builds a response message map without encoding to JSON.\n\nThis is useful for batch processing where we need the raw map.","title":"Hermes.MCP.Message.build_response/2","ref":"Hermes.MCP.Message.html#build_response/2"},{"type":"function","doc":"* `result` - The result data\n  * `id` - The response ID","title":"Parameters - Hermes.MCP.Message.build_response/2","ref":"Hermes.MCP.Message.html#build_response/2-parameters"},{"type":"function","doc":"iex> Message.build_response(%{\"value\" => 42}, \"req_123\")\n    %{\"jsonrpc\" => \"2.0\", \"result\" => %{\"value\" => 42}, \"id\" => \"req_123\"}","title":"Examples - Hermes.MCP.Message.build_response/2","ref":"Hermes.MCP.Message.html#build_response/2-examples"},{"type":"function","doc":"Decodes raw data (possibly containing multiple messages) into JSON-RPC messages.\n\nHandles both single messages and batch messages (arrays).\n\nReturns either:\n- `{:ok, messages}` where messages is a list of parsed JSON-RPC messages\n- `{:error, reason}` if parsing fails","title":"Hermes.MCP.Message.decode/1","ref":"Hermes.MCP.Message.html#decode/1"},{"type":"function","doc":"Encodes a batch of JSON-RPC messages.\n\nThis function uses Peri schema validation and always returns a JSON array.","title":"Hermes.MCP.Message.encode_batch/2","ref":"Hermes.MCP.Message.html#encode_batch/2"},{"type":"function","doc":"* `messages` - A list of complete JSON-RPC message maps\n  * `batch_schema` - Optional Peri schema for batch validation (defaults to :batch_schema)\n\nReturns `{:ok, encoded_batch}` if successful, or `{:error, reason}` if validation fails.","title":"Parameters - Hermes.MCP.Message.encode_batch/2","ref":"Hermes.MCP.Message.html#encode_batch/2-parameters"},{"type":"function","doc":"Encodes an error message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","title":"Hermes.MCP.Message.encode_error/2","ref":"Hermes.MCP.Message.html#encode_error/2"},{"type":"function","doc":"Encodes a log message notification to be sent to the client.","title":"Hermes.MCP.Message.encode_log_message/3","ref":"Hermes.MCP.Message.html#encode_log_message/3"},{"type":"function","doc":"* `level` - The log level (debug, info, notice, warning, error, critical, alert, emergency)\n  * `data` - The data to be logged (any JSON-serializable value)\n  * `logger` - Optional name of the logger issuing the message\n\nReturns the encoded notification string with a newline character appended.","title":"Parameters - Hermes.MCP.Message.encode_log_message/3","ref":"Hermes.MCP.Message.html#encode_log_message/3-parameters"},{"type":"function","doc":"Encodes a notification message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","title":"Hermes.MCP.Message.encode_notification/1","ref":"Hermes.MCP.Message.html#encode_notification/1"},{"type":"function","doc":"Encodes a notification message using a custom schema.","title":"Hermes.MCP.Message.encode_notification/2","ref":"Hermes.MCP.Message.html#encode_notification/2"},{"type":"function","doc":"* `notification` - The notification map containing method and params\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","title":"Parameters - Hermes.MCP.Message.encode_notification/2","ref":"Hermes.MCP.Message.html#encode_notification/2-parameters"},{"type":"function","doc":"Encodes a progress notification message to a JSON-RPC 2.0 compliant string.","title":"Hermes.MCP.Message.encode_progress_notification/2","ref":"Hermes.MCP.Message.html#encode_progress_notification/2"},{"type":"function","doc":"* `params` - Map containing progress parameters:\n    * `\"progressToken\"` - The token that was provided in the original request (string or integer)\n    * `\"progress\"` - The current progress value (number) \n    * `\"total\"` - Optional total value for the operation (number)\n    * `\"message\"` - Optional descriptive message (string, for 2025-03-26)\n  * `params_schema` - Optional Peri schema for params validation (defaults to @progress_notif_params_schema)\n\nReturns the encoded string with a newline character appended.","title":"Parameters - Hermes.MCP.Message.encode_progress_notification/2","ref":"Hermes.MCP.Message.html#encode_progress_notification/2-parameters"},{"type":"function","doc":"Legacy function for progress notifications with individual parameters.\n\n**Deprecated**: Prefer using `encode_progress_notification/2` with a params map.\n\nThis function will be removed in a future release. Update your code to use the newer function:\n\n    encode_progress_notification(%{\n      \"progressToken\" => progress_token,\n      \"progress\" => progress,\n      \"total\" => total\n    })","title":"Hermes.MCP.Message.encode_progress_notification/3","ref":"Hermes.MCP.Message.html#encode_progress_notification/3"},{"type":"function","doc":"Encodes a request message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","title":"Hermes.MCP.Message.encode_request/2","ref":"Hermes.MCP.Message.html#encode_request/2"},{"type":"function","doc":"Encodes a request message using a custom schema.","title":"Hermes.MCP.Message.encode_request/3","ref":"Hermes.MCP.Message.html#encode_request/3"},{"type":"function","doc":"* `request` - The request map containing method and params\n  * `id` - The request ID\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","title":"Parameters - Hermes.MCP.Message.encode_request/3","ref":"Hermes.MCP.Message.html#encode_request/3-parameters"},{"type":"function","doc":"Encodes a response message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","title":"Hermes.MCP.Message.encode_response/2","ref":"Hermes.MCP.Message.html#encode_response/2"},{"type":"function","doc":"Encodes a response message using a custom schema.","title":"Hermes.MCP.Message.encode_response/3","ref":"Hermes.MCP.Message.html#encode_response/3"},{"type":"function","doc":"* `response` - The response map containing result\n  * `id` - The response ID\n  * `schema` - The Peri schema to use for validation\n\nReturns the encoded string with a newline character appended.","title":"Parameters - Hermes.MCP.Message.encode_response/3","ref":"Hermes.MCP.Message.html#encode_response/3-parameters"},{"type":"function","doc":"","title":"Hermes.MCP.Message.error_schema/1","ref":"Hermes.MCP.Message.html#error_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.error_schema!/1","ref":"Hermes.MCP.Message.html#error_schema!/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.get_schema/1","ref":"Hermes.MCP.Message.html#get_schema/1"},{"type":"macro","doc":"Guard to determine if a JSON-RPC message is an error.\n\nA message is considered an error if it contains both \"error\" and \"id\" fields.","title":"Hermes.MCP.Message.is_error/1","ref":"Hermes.MCP.Message.html#is_error/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"error\" => %{\"code\" => -32600}, \"id\" => 1}\n    iex> is_error(message)\n    true","title":"Examples - Hermes.MCP.Message.is_error/1","ref":"Hermes.MCP.Message.html#is_error/1-examples"},{"type":"macro","doc":"Guard to check if a request is an initialize request.","title":"Hermes.MCP.Message.is_initialize/1","ref":"Hermes.MCP.Message.html#is_initialize/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"initialize\", \"id\" => 1, \"params\" => %{}}\n    iex> is_initialize(message)\n    true","title":"Examples - Hermes.MCP.Message.is_initialize/1","ref":"Hermes.MCP.Message.html#is_initialize/1-examples"},{"type":"macro","doc":"Guard to check if a message is part of the initialization lifecycle.\n\nThis includes both the initialize request and the notifications/initialized notification.","title":"Hermes.MCP.Message.is_initialize_lifecycle/1","ref":"Hermes.MCP.Message.html#is_initialize_lifecycle/1"},{"type":"macro","doc":"iex> init_request = %{\"jsonrpc\" => \"2.0\", \"method\" => \"initialize\", \"id\" => 1, \"params\" => %{}}\n    iex> is_initialize_lifecycle(init_request)\n    true\n\n    iex> init_notification = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notifications/initialized\"}\n    iex> is_initialize_lifecycle(init_notification)\n    true\n\n    iex> other_message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"tools/list\", \"id\" => 2}\n    iex> is_initialize_lifecycle(other_message)\n    false","title":"Examples - Hermes.MCP.Message.is_initialize_lifecycle/1","ref":"Hermes.MCP.Message.html#is_initialize_lifecycle/1-examples"},{"type":"macro","doc":"Guard to determine if a JSON-RPC message is a notification.\n\nA message is considered a notification if it contains a \"method\" field but no \"id\" field.","title":"Hermes.MCP.Message.is_notification/1","ref":"Hermes.MCP.Message.html#is_notification/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notification\"}\n    iex> is_notification(message)\n    true\n\n    iex> request = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_notification(request)\n    false","title":"Examples - Hermes.MCP.Message.is_notification/1","ref":"Hermes.MCP.Message.html#is_notification/1-examples"},{"type":"macro","doc":"Guard to check if a request is a ping request.","title":"Hermes.MCP.Message.is_ping/1","ref":"Hermes.MCP.Message.html#is_ping/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_ping(message)\n    true","title":"Examples - Hermes.MCP.Message.is_ping/1","ref":"Hermes.MCP.Message.html#is_ping/1-examples"},{"type":"macro","doc":"Guard to determine if a JSON-RPC message is a request.\n\nA message is considered a request if it contains both \"method\" and \"id\" fields.","title":"Hermes.MCP.Message.is_request/1","ref":"Hermes.MCP.Message.html#is_request/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"method\" => \"ping\", \"id\" => 1}\n    iex> is_request(message)\n    true\n\n    iex> notification = %{\"jsonrpc\" => \"2.0\", \"method\" => \"notification\"}\n    iex> is_request(notification)\n    false","title":"Examples - Hermes.MCP.Message.is_request/1","ref":"Hermes.MCP.Message.html#is_request/1-examples"},{"type":"macro","doc":"Guard to determine if a JSON-RPC message is a response.\n\nA message is considered a response if it contains both \"result\" and \"id\" fields.","title":"Hermes.MCP.Message.is_response/1","ref":"Hermes.MCP.Message.html#is_response/1"},{"type":"macro","doc":"iex> message = %{\"jsonrpc\" => \"2.0\", \"result\" => %{}, \"id\" => 1}\n    iex> is_response(message)\n    true","title":"Examples - Hermes.MCP.Message.is_response/1","ref":"Hermes.MCP.Message.html#is_response/1-examples"},{"type":"function","doc":"","title":"Hermes.MCP.Message.mcp_message_schema/1","ref":"Hermes.MCP.Message.html#mcp_message_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.mcp_message_schema!/1","ref":"Hermes.MCP.Message.html#mcp_message_schema!/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.notification_schema/1","ref":"Hermes.MCP.Message.html#notification_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.notification_schema!/1","ref":"Hermes.MCP.Message.html#notification_schema!/1"},{"type":"function","doc":"Returns the standard progress notification parameters schema for 2024-11-05.","title":"Hermes.MCP.Message.progress_params_schema/0","ref":"Hermes.MCP.Message.html#progress_params_schema/0"},{"type":"function","doc":"Returns the progress notification parameters schema for 2025-03-26 (with message field).","title":"Hermes.MCP.Message.progress_params_schema_2025/0","ref":"Hermes.MCP.Message.html#progress_params_schema_2025/0"},{"type":"function","doc":"","title":"Hermes.MCP.Message.request_schema/1","ref":"Hermes.MCP.Message.html#request_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.request_schema!/1","ref":"Hermes.MCP.Message.html#request_schema!/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.response_schema/1","ref":"Hermes.MCP.Message.html#response_schema/1"},{"type":"function","doc":"","title":"Hermes.MCP.Message.response_schema!/1","ref":"Hermes.MCP.Message.html#response_schema!/1"},{"type":"function","doc":"Validates a decoded JSON message to ensure it complies with the MCP schema.","title":"Hermes.MCP.Message.validate_message/1","ref":"Hermes.MCP.Message.html#validate_message/1"},{"type":"module","doc":"Represents successful responses in the MCP protocol.\n\nThis module provides a wrapper around JSON-RPC responses, handling\ndomain-specific error semantics for MCP's \"isError\" field in results.","title":"Hermes.MCP.Response","ref":"Hermes.MCP.Response.html"},{"type":"module","doc":"Each response includes:\n- `result`: The response data from the server\n- `id`: The request ID this response is associated with\n- `is_error`: Boolean flag indicating if this is a domain-level error","title":"Response Structure - Hermes.MCP.Response","ref":"Hermes.MCP.Response.html#module-response-structure"},{"type":"module","doc":"The MCP protocol distinguishes between two types of errors:\n\n1. Protocol errors: Standard JSON-RPC errors with error codes (handled by `Hermes.MCP.Error`)\n2. Domain errors: Valid responses that indicate application-level errors with `isError: true`\n\nThis module specifically handles domain errors, which are successful at the protocol level\nbut indicate failures at the application level.","title":"Domain vs. Protocol Errors - Hermes.MCP.Response","ref":"Hermes.MCP.Response.html#module-domain-vs-protocol-errors"},{"type":"module","doc":"```elixir\n# Create from a JSON-RPC response\nresponse = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n\n# Check if the response is successful or has a domain error\nif Hermes.MCP.Response.success?(response) do\n  # Handle success\nelse\n  # Handle domain error\nend\n\n# Unwrap the response to get the result or error\ncase Hermes.MCP.Response.unwrap(response) do\n  {:ok, result} -> # Handle success\n  {:error, error} -> # Handle domain error\nend\n```","title":"Examples - Hermes.MCP.Response","ref":"Hermes.MCP.Response.html#module-examples"},{"type":"function","doc":"Checks if the response has a domain error.","title":"Hermes.MCP.Response.error?/1","ref":"Hermes.MCP.Response.html#error?/1"},{"type":"function","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.error?(response)\n    false\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.error?(error_response)\n    true","title":"Examples - Hermes.MCP.Response.error?/1","ref":"Hermes.MCP.Response.html#error?/1-examples"},{"type":"function","doc":"Creates a Response struct from a JSON-RPC response.\n\nAutomatically detects domain errors by checking for the \"isError\" field.","title":"Hermes.MCP.Response.from_json_rpc/1","ref":"Hermes.MCP.Response.html#from_json_rpc/1"},{"type":"function","doc":"* `response` - A map containing the JSON-RPC response","title":"Parameters - Hermes.MCP.Response.from_json_rpc/1","ref":"Hermes.MCP.Response.html#from_json_rpc/1-parameters"},{"type":"function","doc":"iex> Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{}, \"id\" => \"req_123\"})\n    %Hermes.MCP.Response{result: %{}, id: \"req_123\", is_error: false}\n    \n    iex> Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    %Hermes.MCP.Response{result: %{\"isError\" => true}, id: \"req_123\", is_error: true}","title":"Examples - Hermes.MCP.Response.from_json_rpc/1","ref":"Hermes.MCP.Response.html#from_json_rpc/1-examples"},{"type":"function","doc":"Gets the request ID associated with this response.","title":"Hermes.MCP.Response.get_id/1","ref":"Hermes.MCP.Response.html#get_id/1"},{"type":"function","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.get_id(response)\n    \"req_123\"","title":"Examples - Hermes.MCP.Response.get_id/1","ref":"Hermes.MCP.Response.html#get_id/1-examples"},{"type":"function","doc":"Gets the result data from the response.\n\nThis function returns the raw result regardless of whether it represents\na success or domain error.","title":"Hermes.MCP.Response.get_result/1","ref":"Hermes.MCP.Response.html#get_result/1"},{"type":"function","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.get_result(response)\n    %{\"data\" => \"value\"}","title":"Examples - Hermes.MCP.Response.get_result/1","ref":"Hermes.MCP.Response.html#get_result/1-examples"},{"type":"function","doc":"Checks if the response is successful (no domain error).","title":"Hermes.MCP.Response.success?/1","ref":"Hermes.MCP.Response.html#success?/1"},{"type":"function","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.success?(response)\n    true\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.success?(error_response)\n    false","title":"Examples - Hermes.MCP.Response.success?/1","ref":"Hermes.MCP.Response.html#success?/1-examples"},{"type":"type","doc":"","title":"Hermes.MCP.Response.t/0","ref":"Hermes.MCP.Response.html#t:t/0"},{"type":"function","doc":"Unwraps the response, returning the raw result.\n\nReturns the raw result data regardless of whether it represents\na success or domain error.","title":"Hermes.MCP.Response.unwrap/1","ref":"Hermes.MCP.Response.html#unwrap/1"},{"type":"function","doc":"iex> response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"data\" => \"value\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.unwrap(response)\n    %{\"data\" => \"value\"}\n    \n    iex> error_response = Hermes.MCP.Response.from_json_rpc(%{\"result\" => %{\"isError\" => true, \"reason\" => \"not_found\"}, \"id\" => \"req_123\"})\n    iex> Hermes.MCP.Response.unwrap(error_response)\n    %{\"isError\" => true, \"reason\" => \"not_found\"}","title":"Examples - Hermes.MCP.Response.unwrap/1","ref":"Hermes.MCP.Response.html#unwrap/1-examples"},{"type":"behaviour","doc":"Build MCP servers that extend language model capabilities.\n\nMCP servers are specialized processes that provide three core primitives to AI assistants:\n**Resources** (contextual data like files or schemas), **Tools** (actions the model can invoke),\nand **Prompts** (user-selectable templates). They operate in a secure, isolated architecture\nwhere clients maintain 1:1 connections with servers, enabling composable functionality while\nmaintaining strict security boundaries.","title":"Hermes.Server","ref":"Hermes.Server.html"},{"type":"behaviour","doc":"Create a server in three steps:\n\n    defmodule MyServer do\n      use Hermes.Server,\n        name: \"my-server\",\n        version: \"1.0.0\",\n        capabilities: [:tools]\n\n      component MyServer.Calculator\n    end\n\n    defmodule MyServer.Calculator do\n      use Hermes.Server.Component, type: :tool\n\n      def definition do\n        %{\n          name: \"add\",\n          description: \"Add two numbers\",\n          input_schema: %{\n            type: \"object\",\n            properties: %{\n              a: %{type: \"number\"},\n              b: %{type: \"number\"}\n            }\n          }\n        }\n      end\n\n      def call(%{\"a\" => a, \"b\" => b}), do: {:ok, a + b}\n    end\n\n    # Start your server\n    {:ok, _pid} = Hermes.Server.start_link(MyServer, [], transport: :stdio)\n\nYour server is now a living process that AI assistants can connect to, discover available\ntools, and execute calculations through a secure protocol boundary.","title":"Quick Start - Hermes.Server","ref":"Hermes.Server.html#module-quick-start"},{"type":"behaviour","doc":"Declare what your server can do:\n\n- **`:tools`** - Execute functions with structured inputs and outputs\n- **`:resources`** - Provide data that models can read (files, APIs, databases)\n- **`:prompts`** - Offer reusable templates for common interactions\n- **`:logging`** - Allow clients to configure verbosity levels\n\nConfigure capabilities with options:\n\n    use Hermes.Server,\n      capabilities: [\n        :tools,\n        {:resources, subscribe?: true},      # Enable resource update subscriptions\n        {:prompts, list_changed?: true}      # Notify when prompts change\n      ]","title":"Capabilities - Hermes.Server","ref":"Hermes.Server.html#module-capabilities"},{"type":"behaviour","doc":"Register tools, resources, and prompts as components:\n\n    component MyServer.FileReader           # Auto-named as \"file_reader\"\n    component MyServer.ApiClient, name: \"api\"   # Custom name\n\nComponents are modules that implement specific behaviors\nand are automatically discovered by clients through the protocol.","title":"Components - Hermes.Server","ref":"Hermes.Server.html#module-components"},{"type":"behaviour","doc":"Your server follows a predictable lifecycle with callbacks you can hook into:\n\n1. **`init/2`** - Set up initial state when the server starts\n2. **`handle_request/2`** - Process MCP protocol requests from clients\n3. **`handle_notification/2`** - React to one-way client messages\n4. **`handle_info/2`** - Bridge external events into MCP notifications\n\nMost protocol handling is automatic - you typically only implement `init/2` for setup\nand occasionally override other callbacks for custom behavior.","title":"Server Lifecycle - Hermes.Server","ref":"Hermes.Server.html#module-server-lifecycle"},{"type":"macro","doc":"Registers a component (tool, prompt, or resource) with the server.","title":"Hermes.Server.component/2","ref":"Hermes.Server.html#component/2"},{"type":"macro","doc":"# Register with auto-derived name\n    component MyServer.Tools.Calculator\n    \n    # Register with custom name\n    component MyServer.Tools.FileManager, name: \"files\"","title":"Examples - Hermes.Server.component/2","ref":"Hermes.Server.html#component/2-examples"},{"type":"callback","doc":"Handles synchronous calls to the server process.\n\nThis optional callback allows you to handle custom synchronous calls made to your\nMCP server process using `GenServer.call/2`. This is useful for implementing\nadministrative functions, status queries, or any synchronous operations that\nneed to interact with the server's internal state.\n\nThe callback follows standard GenServer semantics and should return appropriate\nreply tuples. If not implemented, the Base module provides a default implementation\nthat handles standard MCP operations.","title":"Hermes.Server.handle_call/3","ref":"Hermes.Server.html#c:handle_call/3"},{"type":"callback","doc":"Handles asynchronous casts to the server process.\n\nThis optional callback allows you to handle custom asynchronous messages sent to your\nMCP server process using `GenServer.cast/2`. This is useful for fire-and-forget\noperations, background tasks, or any asynchronous operations that don't require\nan immediate response.\n\nThe callback follows standard GenServer semantics. If not implemented, the Base\nmodule provides a default implementation that handles standard MCP operations.","title":"Hermes.Server.handle_cast/2","ref":"Hermes.Server.html#c:handle_cast/2"},{"type":"callback","doc":"Handles non-MCP messages sent to the server process.\n\nWhile `handle_request` and `handle_notification` deal with MCP protocol messages,\nthis callback handles everything else - timer events, messages from other processes,\nsystem signals, and any custom inter-process communication your server needs.\n\nThis is particularly useful for servers that need to react to external events\n(like file system changes or database updates) and notify connected clients through\nMCP notifications. Think of it as the bridge between your Elixir application's\ninternal events and the MCP protocol's notification system.","title":"Hermes.Server.handle_info/2","ref":"Hermes.Server.html#c:handle_info/2"},{"type":"callback","doc":"Handles incoming MCP notifications from clients.\n\nNotifications are one-way messages in the MCP protocol - the client informs the server\nabout events or state changes without expecting a response. This fire-and-forget pattern\nis perfect for status updates, progress tracking, and lifecycle events.\n\n**Standard MCP Notifications from Clients:**\n- `notifications/initialized` - Client signals it's ready after successful initialization\n- `notifications/cancelled` - Client requests cancellation of an in-progress operation\n- `notifications/progress` - Client reports progress on a long-running operation\n- `notifications/roots/list_changed` - Client's available filesystem roots have changed\n\nUnlike requests, notifications never receive responses. Any errors during processing\nare typically logged but not communicated back to the client. This makes notifications\nideal for optional features like progress tracking where delivery isn't guaranteed.\n\nThe server processes these notifications to update its internal state, trigger side effects,\nor coordinate with other parts of the system. When using `use Hermes.Server`, basic\nnotification handling is provided, but you'll often want to override this callback\nto handle progress updates or cancellations specific to your server's operations.","title":"Hermes.Server.handle_notification/2","ref":"Hermes.Server.html#c:handle_notification/2"},{"type":"callback","doc":"Handles a prompt get request.\n\nThis callback is invoked when a client requests a specific prompt template. It receives\nthe prompt name, any arguments to fill into the template, and the current frame.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_prompt/3`). For module-based prompts,\nthe framework automatically generates pattern-matched clauses during compilation.","title":"Hermes.Server.handle_prompt_get/3","ref":"Hermes.Server.html#c:handle_prompt_get/3"},{"type":"callback","doc":"Low-level handler for any MCP request.\n\nThis is an advanced callback that gives you complete control over request handling.\nWhen implemented, it bypasses the automatic routing to `handle_tool_call/3`,\n`handle_resource_read/2`, and `handle_prompt_get/3` and all other requests that are\nhandled internally, like `tools/list` and `logging/setLevel`.\n\nUse this when you need to:\n- Implement custom request methods beyond the standard MCP protocol\n- Add middleware-like processing before requests reach specific handlers\n- Override the framework's default request routing behavior\n\nNote: If you implement this callback, you become responsible for handling ALL\nMCP requests, including standard protocol methods like `tools/list`, `resources/list`, etc.\nConsider using the specific callbacks instead unless you need this level of control.","title":"Hermes.Server.handle_request/2","ref":"Hermes.Server.html#c:handle_request/2"},{"type":"callback","doc":"Handles a resource read request.\n\nThis callback is invoked when a client requests to read a specific resource. It receives\nthe resource URI and the current frame. Developer's implementation should retrieve and return\nthe resource content.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_resource/3`). For module-based resources,\nthe framework automatically generates pattern-matched clauses during compilation.","title":"Hermes.Server.handle_resource_read/2","ref":"Hermes.Server.html#c:handle_resource_read/2"},{"type":"callback","doc":"Handles a tool call request.\n\nThis callback is invoked when a client calls a specific tool. It receives the tool name,\nthe arguments provided by the client, and the current frame. Developers's implementation should\nexecute the tool's logic and return the result.\n\nThis callback handles both module-based components (registered with `component`) and\nruntime components (registered with `Frame.register_tool/3`). For module-based tools,\nthe framework automatically generates pattern-matched clauses during compilation.","title":"Hermes.Server.handle_tool_call/3","ref":"Hermes.Server.html#c:handle_tool_call/3"},{"type":"callback","doc":"Called after a client requests a `initialize` request.\n\nThis callback is invoked while the MCP handshake starts and so the client may not sent\nthe `notifications/initialized` message yet. For checking if the notification was already sent\nand the MCP handshare was successfully completed, you can call the `initialized?/1` function.\n\nIt receives the client's information and\nthe current frame, allowing you to perform client-specific setup, validate capabilities,\nor prepare resources based on the connected client.\n\nThe client_info parameter contains details about the connected client including its\nname, version, and any additional metadata. Use this to tailor your server's behavior\nto specific client implementations or versions.","title":"Hermes.Server.init/2","ref":"Hermes.Server.html#c:init/2"},{"type":"function","doc":"Checks if the MCP session has been initialized.\n\nReturns true if the client has completed the initialization handshake and sent\nthe `notifications/initialized` message. This is useful for guarding operations\nthat require an active session.","title":"Hermes.Server.initialized?/1","ref":"Hermes.Server.html#initialized?/1"},{"type":"function","doc":"def handle_info(:check_status, frame) do\n      if Hermes.Server.initialized?(frame) do\n        # Perform operations requiring initialized session\n        {:noreply, frame}\n      else\n        # Wait for initialization\n        {:noreply, frame}\n      end\n    end","title":"Examples - Hermes.Server.initialized?/1","ref":"Hermes.Server.html#initialized?/1-examples"},{"type":"type","doc":"","title":"Hermes.Server.mcp_error/0","ref":"Hermes.Server.html#t:mcp_error/0"},{"type":"type","doc":"","title":"Hermes.Server.notification/0","ref":"Hermes.Server.html#t:notification/0"},{"type":"type","doc":"","title":"Hermes.Server.progress_step/0","ref":"Hermes.Server.html#t:progress_step/0"},{"type":"type","doc":"","title":"Hermes.Server.progress_token/0","ref":"Hermes.Server.html#t:progress_token/0"},{"type":"type","doc":"","title":"Hermes.Server.progress_total/0","ref":"Hermes.Server.html#t:progress_total/0"},{"type":"type","doc":"","title":"Hermes.Server.request/0","ref":"Hermes.Server.html#t:request/0"},{"type":"type","doc":"","title":"Hermes.Server.response/0","ref":"Hermes.Server.html#t:response/0"},{"type":"function","doc":"Sends a log message to the client.\n\nUse this to send diagnostic or informational messages to the client's logging system.","title":"Hermes.Server.send_log_message/4","ref":"Hermes.Server.html#send_log_message/4"},{"type":"function","doc":"Sends a progress notification for an ongoing operation.\n\nUse this to update the client on the progress of long-running operations.","title":"Hermes.Server.send_progress/4","ref":"Hermes.Server.html#send_progress/4"},{"type":"function","doc":"Sends a prompts list changed notification to connected clients.\n\nUse this when the available prompts have changed (added, removed, or modified).\nThe client will typically re-fetch the prompt list in response.","title":"Hermes.Server.send_prompts_list_changed/1","ref":"Hermes.Server.html#send_prompts_list_changed/1"},{"type":"function","doc":"Sends a resource updated notification for a specific resource.\n\nUse this when the content of a specific resource has changed.\nClients that have subscribed to this resource will be notified.","title":"Hermes.Server.send_resource_updated/3","ref":"Hermes.Server.html#send_resource_updated/3"},{"type":"function","doc":"Sends a resources list changed notification to connected clients.\n\nUse this when the available resources have changed (added, removed, or modified).\nThe client will typically re-fetch the resource list in response.","title":"Hermes.Server.send_resources_list_changed/1","ref":"Hermes.Server.html#send_resources_list_changed/1"},{"type":"function","doc":"Sends a tools list changed notification to connected clients.\n\nUse this when the available tools have changed (added, removed, or modified).\nThe client will typically re-fetch the tool list in response.","title":"Hermes.Server.send_tools_list_changed/1","ref":"Hermes.Server.html#send_tools_list_changed/1"},{"type":"callback","doc":"Declares the server's capabilities during initialization.\n\nThis callback tells clients what features your server supports - which types of resources\nit can provide, what tools it can execute, whether it supports logging configuration, etc.\nThe capabilities you declare here directly impact which requests the client will send.\n\nWhen using `use Hermes.Server` with the `capabilities` option, this callback is automatically\nimplemented based on your configuration. The macro analyzes your registered components and\nbuilds the appropriate capability map, so you rarely need to implement this manually.","title":"Hermes.Server.server_capabilities/0","ref":"Hermes.Server.html#c:server_capabilities/0"},{"type":"type","doc":"","title":"Hermes.Server.server_capabilities/0","ref":"Hermes.Server.html#t:server_capabilities/0"},{"type":"callback","doc":"Provides the server's identity information during initialization.\n\nThis callback is called during the MCP handshake to identify your server to connecting clients.\nThe information returned here helps clients understand which server they're talking to and\nensures version compatibility.\n\nWhen using `use Hermes.Server`, this callback is automatically implemented using the\n`name` and `version` options you provide. You only need to implement this manually if\nyou require dynamic server information based on runtime conditions.","title":"Hermes.Server.server_info/0","ref":"Hermes.Server.html#c:server_info/0"},{"type":"type","doc":"","title":"Hermes.Server.server_info/0","ref":"Hermes.Server.html#t:server_info/0"},{"type":"callback","doc":"Specifies which MCP protocol versions this server can speak.\n\nProtocol version negotiation ensures client and server can communicate effectively.\nDuring initialization, the client and server agree on a mutually supported version.\nThis callback returns the list of versions your server understands, typically in\norder of preference from newest to oldest.\n\nWhen using `use Hermes.Server`, this is automatically implemented with sensible defaults\ncovering current and recent protocol versions. Override only if you need to restrict\nor extend version support for specific compatibility requirements.","title":"Hermes.Server.supported_protocol_versions/0","ref":"Hermes.Server.html#c:supported_protocol_versions/0"},{"type":"callback","doc":"Cleans up when the server process terminates.\n\nThis optional callback is invoked when the server process is about to terminate.\nIt allows you to perform cleanup operations, close connections, save state,\nor release resources before the process exits.\n\nThe callback receives the termination reason and the current frame. Any return\nvalue is ignored. If not implemented, the Base module provides a default\nimplementation that logs the termination event.","title":"Hermes.Server.terminate/2","ref":"Hermes.Server.html#c:terminate/2"},{"type":"behaviour","doc":"Defines the behaviour for MCP prompts.\n\nPrompts are reusable templates that generate messages based on provided arguments.\nThey help standardize common interactions and can be customized with parameters.","title":"Hermes.Server.Component.Prompt","ref":"Hermes.Server.Component.Prompt.html"},{"type":"behaviour","doc":"defmodule MyServer.Prompts.CodeReview do\n      @behaviour Hermes.Server.Behaviour.Prompt\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def name, do: \"code_review\"\n      \n      @impl true\n      def description do\n        \"Generate a code review prompt for the given programming language and code\"\n      end\n      \n      @impl true\n      def arguments do\n        [\n          %{\n            \"name\" => \"language\",\n            \"description\" => \"The programming language of the code\",\n            \"required\" => true\n          },\n          %{\n            \"name\" => \"code\",\n            \"description\" => \"The code to review\",\n            \"required\" => true\n          },\n          %{\n            \"name\" => \"focus_areas\",\n            \"description\" => \"Specific areas to focus on (e.g., performance, security)\",\n            \"required\" => false\n          }\n        ]\n      end\n      \n      @impl true\n      def get_messages(%{\"language\" => lang, \"code\" => code} = args, frame) do\n        focus = Map.get(args, \"focus_areas\", \"general quality\")\n        \n        messages = [\n          %{\n            \"role\" => \"user\",\n            \"content\" => %{\n              \"type\" => \"text\",\n              \"text\" => \"\"\"\n              Please review the following #{lang} code, focusing on #{focus}:\n              \n              ```#{lang}\n              #{code}\n              ```\n              \n              Provide constructive feedback on:\n              1. Code quality and readability\n              2. Potential bugs or issues\n              3. Performance considerations\n              4. Best practices for #{lang}\n              \"\"\"\n            }\n          }\n        ]\n        \n        # Can track prompt usage\n        new_frame = Frame.assign(frame, :last_prompt_used, \"code_review\")\n        \n        {:ok, messages, new_frame}\n      end\n    end","title":"Example - Hermes.Server.Component.Prompt","ref":"Hermes.Server.Component.Prompt.html#module-example"},{"type":"type","doc":"","title":"Hermes.Server.Component.Prompt.argument_def/0","ref":"Hermes.Server.Component.Prompt.html#t:argument_def/0"},{"type":"callback","doc":"Returns the list of arguments this prompt accepts.\n\nEach argument should include:\n- `\"name\"` - The argument name\n- `\"description\"` - What the argument is for\n- `\"required\"` - Whether the argument is required (optional, defaults to false)","title":"Hermes.Server.Component.Prompt.arguments/0","ref":"Hermes.Server.Component.Prompt.html#c:arguments/0"},{"type":"callback","doc":"[\n      %{\n        \"name\" => \"topic\",\n        \"description\" => \"The topic to generate content about\",\n        \"required\" => true\n      },\n      %{\n        \"name\" => \"tone\",\n        \"description\" => \"The tone of voice (formal, casual, etc.)\",\n        \"required\" => false\n      }\n    ]","title":"Example - Hermes.Server.Component.Prompt.arguments/0","ref":"Hermes.Server.Component.Prompt.html#c:arguments/0-example"},{"type":"type","doc":"","title":"Hermes.Server.Component.Prompt.arguments/0","ref":"Hermes.Server.Component.Prompt.html#t:arguments/0"},{"type":"callback","doc":"Generates messages based on the provided arguments.","title":"Hermes.Server.Component.Prompt.get_messages/2","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2"},{"type":"callback","doc":"- `args` - The arguments provided by the client\n- `frame` - The server frame containing context and state","title":"Parameters - Hermes.Server.Component.Prompt.get_messages/2","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-parameters"},{"type":"callback","doc":"- `{:ok, messages}` - Messages generated successfully, frame unchanged\n- `{:ok, messages, new_frame}` - Messages generated with frame updates\n- `{:error, reason}` - Failed to generate messages","title":"Return Values - Hermes.Server.Component.Prompt.get_messages/2","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-return-values"},{"type":"callback","doc":"Messages should follow the MCP message format:\n\n    %{\n      \"role\" => \"user\" | \"assistant\",\n      \"content\" => %{\n        \"type\" => \"text\",\n        \"text\" => \"The message content\"\n      }\n    }\n\nMultiple messages can be returned to create a conversation context.","title":"Message Format - Hermes.Server.Component.Prompt.get_messages/2","ref":"Hermes.Server.Component.Prompt.html#c:get_messages/2-message-format"},{"type":"type","doc":"","title":"Hermes.Server.Component.Prompt.message/0","ref":"Hermes.Server.Component.Prompt.html#t:message/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Prompt.t/0","ref":"Hermes.Server.Component.Prompt.html#t:t/0"},{"type":"behaviour","doc":"Defines the behaviour for MCP resources.\n\nResources represent data that can be read by the client, such as files,\ndocuments, or any other content. Each resource is identified by a URI\nand can provide content in various formats.","title":"Hermes.Server.Component.Resource","ref":"Hermes.Server.Component.Resource.html"},{"type":"behaviour","doc":"defmodule MyServer.Resources.Documentation do\n      @behaviour Hermes.Server.Behaviour.Resource\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def uri, do: \"file:///docs/readme.md\"\n      \n      @impl true\n      def name, do: \"Project README\"\n      \n      @impl true\n      def description, do: \"The main documentation for this project\"\n      \n      @impl true\n      def mime_type, do: \"text/markdown\"\n      \n      @impl true\n      def read(_params, frame) do\n        case File.read(\"README.md\") do\n          {:ok, content} -> \n            # Can track access in frame\n            new_frame = Frame.assign(frame, :last_resource_access, DateTime.utc_now())\n            {:ok, content, new_frame}\n            \n          {:error, reason} -> \n            {:error, \"Failed to read README: #{inspect(reason)}\"}\n        end\n      end\n    end","title":"Example - Hermes.Server.Component.Resource","ref":"Hermes.Server.Component.Resource.html#module-example"},{"type":"behaviour","doc":"defmodule MyServer.Resources.SystemStatus do\n      @behaviour Hermes.Server.Behaviour.Resource\n      \n      @impl true\n      def uri, do: \"system://status\"\n      \n      @impl true\n      def name, do: \"System Status\"\n      \n      @impl true\n      def description, do: \"Current system status and metrics\"\n      \n      @impl true\n      def mime_type, do: \"application/json\"\n      \n      @impl true\n      def read(_params, frame) do\n        status = %{\n          uptime: System.uptime(),\n          memory: :erlang.memory(),\n          user_id: frame.assigns[:user_id],\n          timestamp: DateTime.utc_now()\n        }\n        \n        {:ok, Jason.encode!(status), frame}\n      end\n    end","title":"Example with dynamic content - Hermes.Server.Component.Resource","ref":"Hermes.Server.Component.Resource.html#module-example-with-dynamic-content"},{"type":"type","doc":"","title":"Hermes.Server.Component.Resource.content/0","ref":"Hermes.Server.Component.Resource.html#t:content/0"},{"type":"callback","doc":"Returns the MIME type of the resource content.\n\nCommon MIME types:\n- `text/plain` for plain text\n- `text/markdown` for Markdown\n- `application/json` for JSON data\n- `text/html` for HTML\n- `application/octet-stream` for binary data","title":"Hermes.Server.Component.Resource.mime_type/0","ref":"Hermes.Server.Component.Resource.html#c:mime_type/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Resource.params/0","ref":"Hermes.Server.Component.Resource.html#t:params/0"},{"type":"callback","doc":"Reads the resource content.","title":"Hermes.Server.Component.Resource.read/2","ref":"Hermes.Server.Component.Resource.html#c:read/2"},{"type":"callback","doc":"- `params` - Optional parameters from the client (typically empty for resources)\n- `frame` - The server frame containing context and state","title":"Parameters - Hermes.Server.Component.Resource.read/2","ref":"Hermes.Server.Component.Resource.html#c:read/2-parameters"},{"type":"callback","doc":"- `{:ok, content}` - Resource read successfully, frame unchanged\n- `{:ok, content, new_frame}` - Resource read successfully with frame updates\n- `{:error, reason}` - Failed to read resource","title":"Return Values - Hermes.Server.Component.Resource.read/2","ref":"Hermes.Server.Component.Resource.html#c:read/2-return-values"},{"type":"callback","doc":"The content should match the declared MIME type:\n- For text types, return a String\n- For binary types, return binary data\n- For JSON, return the JSON-encoded string","title":"Content Types - Hermes.Server.Component.Resource.read/2","ref":"Hermes.Server.Component.Resource.html#c:read/2-content-types"},{"type":"type","doc":"","title":"Hermes.Server.Component.Resource.t/0","ref":"Hermes.Server.Component.Resource.html#t:t/0"},{"type":"callback","doc":"Returns the URI that identifies this resource.\n\nThe URI should be unique within the server and follow standard URI conventions.\nCommon schemes include:\n- `file://` for file-based resources\n- `http://` or `https://` for web resources\n- Custom schemes for application-specific resources","title":"Hermes.Server.Component.Resource.uri/0","ref":"Hermes.Server.Component.Resource.html#c:uri/0"},{"type":"behaviour","doc":"Defines the behaviour for MCP tools.\n\nTools are functions that can be invoked by the client with specific parameters.\nEach tool must define its name, description, and parameter schema, as well as\nimplement the execution logic.","title":"Hermes.Server.Component.Tool","ref":"Hermes.Server.Component.Tool.html"},{"type":"behaviour","doc":"defmodule MyServer.Tools.Calculator do\n      @behaviour Hermes.Server.Behaviour.Tool\n      \n      alias Hermes.Server.Frame\n      \n      @impl true\n      def name, do: \"calculator\"\n      \n      @impl true\n      def description, do: \"Performs basic arithmetic operations\"\n      \n      @impl true\n      def input_schema do\n        %{\n          \"type\" => \"object\",\n          \"properties\" => %{\n            \"operation\" => %{\n              \"type\" => \"string\",\n              \"enum\" => [\"add\", \"subtract\", \"multiply\", \"divide\"]\n            },\n            \"a\" => %{\"type\" => \"number\"},\n            \"b\" => %{\"type\" => \"number\"}\n          },\n          \"required\" => [\"operation\", \"a\", \"b\"]\n        }\n      end\n      \n      @impl true\n      def execute(%{\"operation\" => \"add\", \"a\" => a, \"b\" => b}, frame) do\n        result = a + b\n        \n        # Can access frame assigns\n        user_id = frame.assigns[:user_id]\n        \n        # Can return updated frame if needed\n        new_frame = Frame.assign(frame, :last_calculation, result)\n        \n        {:ok, result, new_frame}\n      end\n      \n      @impl true\n      def execute(%{\"operation\" => \"divide\", \"a\" => a, \"b\" => 0}, _frame) do\n        {:error, \"Cannot divide by zero\"}\n      end\n    end","title":"Example - Hermes.Server.Component.Tool","ref":"Hermes.Server.Component.Tool.html#module-example"},{"type":"callback","doc":"Returns optional annotations for the tool.\n\nAnnotations provide additional metadata about the tool that may be used\nby clients for enhanced functionality. This is an optional callback.","title":"Hermes.Server.Component.Tool.annotations/0","ref":"Hermes.Server.Component.Tool.html#c:annotations/0"},{"type":"callback","doc":"def annotations do\n      %{\n        \"confidence\" => 0.95,\n        \"category\" => \"text-processing\",\n        \"tags\" => [\"nlp\", \"text\"]\n      }\n    end","title":"Examples - Hermes.Server.Component.Tool.annotations/0","ref":"Hermes.Server.Component.Tool.html#c:annotations/0-examples"},{"type":"type","doc":"","title":"Hermes.Server.Component.Tool.annotations/0","ref":"Hermes.Server.Component.Tool.html#t:annotations/0"},{"type":"callback","doc":"Executes the tool with the given parameters.","title":"Hermes.Server.Component.Tool.execute/2","ref":"Hermes.Server.Component.Tool.html#c:execute/2"},{"type":"callback","doc":"- `params` - The validated input parameters from the client\n- `frame` - The server frame containing:\n  - `assigns` - Custom data like session_id, client_info, user permissions\n  - `initialized` - Whether the server has been initialized","title":"Parameters - Hermes.Server.Component.Tool.execute/2","ref":"Hermes.Server.Component.Tool.html#c:execute/2-parameters"},{"type":"callback","doc":"- `{:ok, result}` - Tool executed successfully, frame unchanged\n- `{:ok, result, new_frame}` - Tool executed successfully with frame updates\n- `{:error, reason}` - Tool failed with the given reason","title":"Return Values - Hermes.Server.Component.Tool.execute/2","ref":"Hermes.Server.Component.Tool.html#c:execute/2-return-values"},{"type":"callback","doc":"The frame provides access to server state and context:\n\n    def execute(params, frame) do\n      # Access assigns\n      user_id = frame.assigns[:user_id]\n      permissions = frame.assigns[:permissions]\n      \n      # Update frame if needed\n      new_frame = Frame.assign(frame, :last_tool_call, DateTime.utc_now())\n      \n      {:ok, \"Result\", new_frame}\n    end","title":"Frame Usage - Hermes.Server.Component.Tool.execute/2","ref":"Hermes.Server.Component.Tool.html#c:execute/2-frame-usage"},{"type":"callback","doc":"Returns the JSON Schema for the tool's input parameters.\n\nThis schema is used to validate client requests and generate documentation.\nThe schema should follow the JSON Schema specification.","title":"Hermes.Server.Component.Tool.input_schema/0","ref":"Hermes.Server.Component.Tool.html#c:input_schema/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Tool.params/0","ref":"Hermes.Server.Component.Tool.html#t:params/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Tool.result/0","ref":"Hermes.Server.Component.Tool.html#t:result/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Tool.schema/0","ref":"Hermes.Server.Component.Tool.html#t:schema/0"},{"type":"type","doc":"","title":"Hermes.Server.Component.Tool.t/0","ref":"Hermes.Server.Component.Tool.html#t:t/0"},{"type":"module","doc":"The Hermes Frame.\n\nThis module defines a struct and functions for working with\nMCP server state throughout the request/response lifecycle.","title":"Hermes.Server.Frame","ref":"Hermes.Server.Frame.html"},{"type":"module","doc":"These fields contain user-controlled data:\n\n  * `assigns` - shared user data as a map. For HTTP transports, this inherits\n    from `Plug.Conn.assigns`. Users are responsible for populating authentication\n    data through their Plug pipeline before it reaches the MCP server.","title":"User fields - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-user-fields"},{"type":"module","doc":"These fields contain transport-specific context. The structure varies by transport type:","title":"Transport fields - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-transport-fields"},{"type":"module","doc":"* `req_headers` - the request headers as a list, example: `[{\"content-type\", \"application/json\"}]`.\n    All header names are downcased.\n  * `query_params` - the request query params as a map, example: `%{\"session\" => \"abc123\"}`.\n    Returns `nil` if query params were not fetched by the Plug pipeline.\n  * `remote_ip` - the IP of the client, example: `{151, 236, 219, 228}`. \n    This field is set by the transport layer.\n  * `scheme` - the request scheme as an atom, example: `:https`\n  * `host` - the requested host as a binary, example: `\"api.example.com\"`\n  * `port` - the requested port as an integer, example: `443`\n  * `request_path` - the requested path, example: `\"/mcp\"`","title":"HTTP transport (when transport.type == :http) - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-http-transport-when-transport-type-http"},{"type":"module","doc":"* `env` - environment variables as a map, example: `%{\"USER\" => \"alice\", \"HOME\" => \"/home/alice\"}`\n  * `pid` - the OS process ID as a string, example: `\"12345\"`","title":"STDIO transport (when transport.type == :stdio) - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-stdio-transport-when-transport-type-stdio"},{"type":"module","doc":"These fields contain MCP-specific data:\n\n  * `request` - the current MCP request being processed, with fields:\n    * `id` - the request ID for correlation\n    * `method` - the MCP method being called, example: `\"tools/call\"`\n    * `params` - the raw request parameters (before validation)\n  * `initialized` - boolean indicating if the MCP session has been initialized","title":"MCP protocol fields - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-mcp-protocol-fields"},{"type":"module","doc":"These fields are reserved for framework usage:\n\n  * `private` - shared framework data as a map. Contains MCP session context:\n    * `session_id` - unique identifier for the current client session being handled\n    * `client_info` - client information from initialization, example: `%{\"name\" => \"my-client\", \"version\" => \"1.0.0\"}`\n    * `client_capabilities` - negotiated client capabilities\n    * `protocol_version` - active MCP protocol version, example: `\"2025-03-26\"`","title":"Private fields - Hermes.Server.Frame","ref":"Hermes.Server.Frame.html#module-private-fields"},{"type":"function","doc":"Assigns a value or multiple values to the frame.","title":"Hermes.Server.Frame.assign/2","ref":"Hermes.Server.Frame.html#assign/2"},{"type":"function","doc":"# Single assignment\n    frame = Frame.assign(frame, :status, :active)\n\n    # Multiple assignments via map\n    frame = Frame.assign(frame, %{status: :active, count: 5})\n\n    # Multiple assignments via keyword list\n    frame = Frame.assign(frame, status: :active, count: 5)","title":"Examples - Hermes.Server.Frame.assign/2","ref":"Hermes.Server.Frame.html#assign/2-examples"},{"type":"function","doc":"","title":"Hermes.Server.Frame.assign/3","ref":"Hermes.Server.Frame.html#assign/3"},{"type":"function","doc":"Assigns a value to the frame only if the key doesn't already exist.\n\nThe value is computed lazily using the provided function, which is only\ncalled if the key is not present in assigns.","title":"Hermes.Server.Frame.assign_new/3","ref":"Hermes.Server.Frame.html#assign_new/3"},{"type":"function","doc":"# Only assigns if :timestamp doesn't exist\n    frame = Frame.assign_new(frame, :timestamp, fn -> DateTime.utc_now() end)\n\n    # Function is not called if key exists\n    frame = frame |> Frame.assign(:count, 5)\n                  |> Frame.assign_new(:count, fn -> expensive_computation() end)\n    # count remains 5","title":"Examples - Hermes.Server.Frame.assign_new/3","ref":"Hermes.Server.Frame.html#assign_new/3-examples"},{"type":"function","doc":"Clears the current request from the frame.\n\nThis should be called after processing a request to ensure the frame doesn't\nretain stale request data.","title":"Hermes.Server.Frame.clear_request/1","ref":"Hermes.Server.Frame.html#clear_request/1"},{"type":"function","doc":"frame = Frame.clear_request(frame)","title":"Examples - Hermes.Server.Frame.clear_request/1","ref":"Hermes.Server.Frame.html#clear_request/1-examples"},{"type":"function","doc":"Clears all session-specific private data from the frame.\n\nThis should be called when a session ends to ensure the frame doesn't\nretain stale session data.","title":"Hermes.Server.Frame.clear_session/1","ref":"Hermes.Server.Frame.html#clear_session/1"},{"type":"function","doc":"frame = Frame.clear_session(frame)","title":"Examples - Hermes.Server.Frame.clear_session/1","ref":"Hermes.Server.Frame.html#clear_session/1-examples"},{"type":"function","doc":"Gets the client capabilities from the frame's private data.","title":"Hermes.Server.Frame.get_client_capabilities/1","ref":"Hermes.Server.Frame.html#get_client_capabilities/1"},{"type":"function","doc":"capabilities = Frame.get_client_capabilities(frame)\n    # => %{\"tools\" => %{}, \"resources\" => %{}}","title":"Examples - Hermes.Server.Frame.get_client_capabilities/1","ref":"Hermes.Server.Frame.html#get_client_capabilities/1-examples"},{"type":"function","doc":"Gets the client info from the frame's private data.","title":"Hermes.Server.Frame.get_client_info/1","ref":"Hermes.Server.Frame.html#get_client_info/1"},{"type":"function","doc":"client_info = Frame.get_client_info(frame)\n    # => %{\"name\" => \"my-client\", \"version\" => \"1.0.0\"}","title":"Examples - Hermes.Server.Frame.get_client_info/1","ref":"Hermes.Server.Frame.html#get_client_info/1-examples"},{"type":"function","doc":"Gets the MCP session ID from the frame's private data.","title":"Hermes.Server.Frame.get_mcp_session_id/1","ref":"Hermes.Server.Frame.html#get_mcp_session_id/1"},{"type":"function","doc":"session_id = Frame.get_mcp_session_id(frame)\n    # => \"session_abc123\"","title":"Examples - Hermes.Server.Frame.get_mcp_session_id/1","ref":"Hermes.Server.Frame.html#get_mcp_session_id/1-examples"},{"type":"function","doc":"Gets the protocol version from the frame's private data.","title":"Hermes.Server.Frame.get_protocol_version/1","ref":"Hermes.Server.Frame.html#get_protocol_version/1"},{"type":"function","doc":"version = Frame.get_protocol_version(frame)\n    # => \"2025-03-26\"","title":"Examples - Hermes.Server.Frame.get_protocol_version/1","ref":"Hermes.Server.Frame.html#get_protocol_version/1-examples"},{"type":"function","doc":"Gets a query parameter value from HTTP transport.\n\nReturns the parameter value, or nil if the transport is not HTTP,\nquery params weren't fetched, or the parameter doesn't exist.","title":"Hermes.Server.Frame.get_query_param/2","ref":"Hermes.Server.Frame.html#get_query_param/2"},{"type":"function","doc":"# HTTP transport with query params\n    session = Frame.get_query_param(frame, \"session\")\n    # => \"abc123\"\n\n    # Missing parameter or non-HTTP transport\n    missing = Frame.get_query_param(frame, \"nonexistent\")\n    # => nil","title":"Examples - Hermes.Server.Frame.get_query_param/2","ref":"Hermes.Server.Frame.html#get_query_param/2-examples"},{"type":"function","doc":"Gets a request header value from HTTP transport.\n\nReturns the first value for the header, or nil if the transport\nis not HTTP or the header is not present.","title":"Hermes.Server.Frame.get_req_header/2","ref":"Hermes.Server.Frame.html#get_req_header/2"},{"type":"function","doc":"# HTTP transport\n    auth_header = Frame.get_req_header(frame, \"authorization\")\n    # => \"Bearer token123\"\n\n    # Non-HTTP transport or missing header\n    auth_header = Frame.get_req_header(frame, \"authorization\")\n    # => nil","title":"Examples - Hermes.Server.Frame.get_req_header/2","ref":"Hermes.Server.Frame.html#get_req_header/2-examples"},{"type":"type","doc":"","title":"Hermes.Server.Frame.http_t/0","ref":"Hermes.Server.Frame.html#t:http_t/0"},{"type":"function","doc":"Creates a new frame with optional initial assigns.","title":"Hermes.Server.Frame.new/1","ref":"Hermes.Server.Frame.html#new/1"},{"type":"function","doc":"iex> Frame.new()\n    %Frame{assigns: %{}, initialized: false}\n\n    iex> Frame.new(%{user: \"alice\"})\n    %Frame{assigns: %{user: \"alice\"}, initialized: false}","title":"Examples - Hermes.Server.Frame.new/1","ref":"Hermes.Server.Frame.html#new/1-examples"},{"type":"type","doc":"","title":"Hermes.Server.Frame.private_t/0","ref":"Hermes.Server.Frame.html#t:private_t/0"},{"type":"function","doc":"","title":"Hermes.Server.Frame.put_private/2","ref":"Hermes.Server.Frame.html#put_private/2"},{"type":"function","doc":"Sets or updates private session data in the frame.\n\nPrivate data is used for framework-internal session context that persists\nacross requests, similar to Plug.Conn.private.","title":"Hermes.Server.Frame.put_private/3","ref":"Hermes.Server.Frame.html#put_private/3"},{"type":"function","doc":"# Set single private value\n    frame = Frame.put_private(frame, :session_id, \"abc123\")\n\n    # Set multiple private values\n    frame = Frame.put_private(frame, %{\n      session_id: \"abc123\",\n      client_info: %{name: \"my-client\", version: \"1.0.0\"}\n    })","title":"Examples - Hermes.Server.Frame.put_private/3","ref":"Hermes.Server.Frame.html#put_private/3-examples"},{"type":"function","doc":"Sets the current request being processed.\n\nThe request includes the request ID, method, and raw parameters before validation.","title":"Hermes.Server.Frame.put_request/2","ref":"Hermes.Server.Frame.html#put_request/2"},{"type":"function","doc":"frame = Frame.put_request(frame, %{\n      id: \"req_123\",\n      method: \"tools/call\",\n      params: %{\"name\" => \"calculator\", \"arguments\" => %{}}\n    })","title":"Examples - Hermes.Server.Frame.put_request/2","ref":"Hermes.Server.Frame.html#put_request/2-examples"},{"type":"function","doc":"","title":"Hermes.Server.Frame.put_transport/2","ref":"Hermes.Server.Frame.html#put_transport/2"},{"type":"function","doc":"Sets or updates transport data in the frame.\n\nCheck `transport_t()` for reference.","title":"Hermes.Server.Frame.put_transport/3","ref":"Hermes.Server.Frame.html#put_transport/3"},{"type":"function","doc":"# Set single transport value\n    frame = Frame.put_transport(frame, :session_id, \"abc123\")\n\n    # Set multiple transport values\n    frame = Frame.put_transport(frame, %{\n      session_id: \"abc123\",\n      client_info: %{name: \"my-client\", version: \"1.0.0\"}\n    })","title":"Examples - Hermes.Server.Frame.put_transport/3","ref":"Hermes.Server.Frame.html#put_transport/3-examples"},{"type":"function","doc":"Registers a prompt definition.","title":"Hermes.Server.Frame.register_prompt/3","ref":"Hermes.Server.Frame.html#register_prompt/3"},{"type":"function","doc":"Registers a resource definition.","title":"Hermes.Server.Frame.register_resource/3","ref":"Hermes.Server.Frame.html#register_resource/3"},{"type":"function","doc":"Registers a tool definition.","title":"Hermes.Server.Frame.register_tool/3","ref":"Hermes.Server.Frame.html#register_tool/3"},{"type":"type","doc":"","title":"Hermes.Server.Frame.request_t/0","ref":"Hermes.Server.Frame.html#t:request_t/0"},{"type":"type","doc":"","title":"Hermes.Server.Frame.server_component_t/0","ref":"Hermes.Server.Frame.html#t:server_component_t/0"},{"type":"type","doc":"","title":"Hermes.Server.Frame.stdio_t/0","ref":"Hermes.Server.Frame.html#t:stdio_t/0"},{"type":"type","doc":"","title":"Hermes.Server.Frame.t/0","ref":"Hermes.Server.Frame.html#t:t/0"},{"type":"type","doc":"","title":"Hermes.Server.Frame.transport_t/0","ref":"Hermes.Server.Frame.html#t:transport_t/0"},{"type":"behaviour","doc":"Behaviour for registry adapters in MCP servers.\n\nThis module defines the interface that registry implementations must follow\nto be pluggable into the Hermes MCP server architecture. It allows users\nto provide custom registry implementations (e.g., using Horde for cluster-wide\ndistribution) while maintaining compatibility with the existing API.","title":"Hermes.Server.Registry.Adapter","ref":"Hermes.Server.Registry.Adapter.html"},{"type":"behaviour","doc":"To implement a custom registry adapter, create a module that implements\nall the callbacks defined in this behaviour:\n\n    defmodule MyApp.HordeRegistry do\n      @behaviour Hermes.Server.Registry.Adapter\n\n      def child_spec(opts) do\n        %{\n          id: __MODULE__,\n          start: {Horde.Registry, :start_link, [\n            [\n              name: __MODULE__,\n              keys: :unique,\n              members: :auto\n            ] ++ opts\n          ]}\n        }\n      end\n\n      def server(module) do\n        {:via, Horde.Registry, {__MODULE__, {:server, module}}}\n      end\n\n      # ... implement other callbacks\n    end","title":"Implementing a Custom Registry - Hermes.Server.Registry.Adapter","ref":"Hermes.Server.Registry.Adapter.html#module-implementing-a-custom-registry"},{"type":"behaviour","doc":"You can configure a custom registry at multiple levels:\n\n  Hermes.Server.start_link(MyServer, :ok, transport: :stdio, registry: MyApp.HordeRegistry)","title":"Using a Custom Registry - Hermes.Server.Registry.Adapter","ref":"Hermes.Server.Registry.Adapter.html#module-using-a-custom-registry"},{"type":"behaviour","doc":"The default implementation uses Elixir's built-in Registry module.","title":"Default Implementation - Hermes.Server.Registry.Adapter","ref":"Hermes.Server.Registry.Adapter.html#module-default-implementation"},{"type":"callback","doc":"Returns a child specification for the registry.\n\nThis is used when starting the registry as part of a supervision tree.\nThe implementation should return a valid child specification map or tuple.","title":"Hermes.Server.Registry.Adapter.child_spec/1","ref":"Hermes.Server.Registry.Adapter.html#c:child_spec/1"},{"type":"callback","doc":"Returns a name for a server process.\n\nThe returned value must be a valid GenServer name that can be passed\nto `GenServer.start_link/3` and similar functions.","title":"Hermes.Server.Registry.Adapter.server/1","ref":"Hermes.Server.Registry.Adapter.html#c:server/1"},{"type":"callback","doc":"* `module` - The module implementing the server","title":"Parameters - Hermes.Server.Registry.Adapter.server/1","ref":"Hermes.Server.Registry.Adapter.html#c:server/1-parameters"},{"type":"callback","doc":"Returns a name for a server session process.","title":"Hermes.Server.Registry.Adapter.server_session/2","ref":"Hermes.Server.Registry.Adapter.html#c:server_session/2"},{"type":"callback","doc":"* `server_module` - The module implementing the server\n  * `session_id` - The unique session identifier","title":"Parameters - Hermes.Server.Registry.Adapter.server_session/2","ref":"Hermes.Server.Registry.Adapter.html#c:server_session/2-parameters"},{"type":"callback","doc":"Returns a name for a supervisor process.","title":"Hermes.Server.Registry.Adapter.supervisor/2","ref":"Hermes.Server.Registry.Adapter.html#c:supervisor/2"},{"type":"callback","doc":"* `kind` - The kind of supervisor (e.g., :supervisor, :session_supervisor)\n  * `server_module` - The module implementing the server","title":"Parameters - Hermes.Server.Registry.Adapter.supervisor/2","ref":"Hermes.Server.Registry.Adapter.html#c:supervisor/2-parameters"},{"type":"callback","doc":"Returns a name for a transport process.","title":"Hermes.Server.Registry.Adapter.transport/2","ref":"Hermes.Server.Registry.Adapter.html#c:transport/2"},{"type":"callback","doc":"* `server_module` - The module implementing the server\n  * `transport_type` - The type of transport (e.g., :stdio, :sse, :websocket)","title":"Parameters - Hermes.Server.Registry.Adapter.transport/2","ref":"Hermes.Server.Registry.Adapter.html#c:transport/2-parameters"},{"type":"callback","doc":"Gets the PID of a registered server.\n\nReturns the PID if the server is registered, nil otherwise.","title":"Hermes.Server.Registry.Adapter.whereis_server/1","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server/1"},{"type":"callback","doc":"* `server_module` - The module implementing the server","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_server/1","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server/1-parameters"},{"type":"callback","doc":"Gets the PID of a server session process.\n\nReturns the PID if the session is registered, nil otherwise.","title":"Hermes.Server.Registry.Adapter.whereis_server_session/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server_session/2"},{"type":"callback","doc":"* `server_module` - The module implementing the server\n  * `session_id` - The unique session identifier","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_server_session/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_server_session/2-parameters"},{"type":"callback","doc":"Gets the PID of a supervisor process.\n\nReturns the PID if the supervisor is registered, nil otherwise.","title":"Hermes.Server.Registry.Adapter.whereis_supervisor/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_supervisor/2"},{"type":"callback","doc":"* `kind` - The kind of supervisor\n  * `server_module` - The module implementing the server","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_supervisor/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_supervisor/2-parameters"},{"type":"callback","doc":"Gets the PID of a transport process.\n\nReturns the PID if the transport is registered, nil otherwise.","title":"Hermes.Server.Registry.Adapter.whereis_transport/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_transport/2"},{"type":"callback","doc":"* `server_module` - The module implementing the server\n  * `transport_type` - The type of transport","title":"Parameters - Hermes.Server.Registry.Adapter.whereis_transport/2","ref":"Hermes.Server.Registry.Adapter.html#c:whereis_transport/2-parameters"},{"type":"module","doc":"Fluent interface for building MCP component responses.\n\nThis module provides builders for tool, prompt, and resource responses\nthat integrate seamlessly with the component system.","title":"Hermes.Server.Response","ref":"Hermes.Server.Response.html"},{"type":"module","doc":"# Tool response\n    Response.tool()\n    |> Response.text(\"Result: \" <> result)\n    |> Response.build()\n    \n    # Resource response (uri and mime_type come from component)\n    Response.resource()\n    |> Response.text(file_contents)\n    |> Response.build()\n    \n    # Prompt response\n    Response.prompt()\n    |> Response.user_message(\"What is the weather?\")\n    |> Response.assistant_message(\"Let me check...\")\n    |> Response.build()","title":"Examples - Hermes.Server.Response","ref":"Hermes.Server.Response.html#module-examples"},{"type":"function","doc":"Add an assistant message to a prompt response.","title":"Hermes.Server.Response.assistant_message/2","ref":"Hermes.Server.Response.html#assistant_message/2"},{"type":"function","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","title":"Parameters - Hermes.Server.Response.assistant_message/2","ref":"Hermes.Server.Response.html#assistant_message/2-parameters"},{"type":"function","doc":"iex> Response.prompt() |> Response.assistant_message(\"Let me check the weather for you.\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"assistant\", \"content\" => \"Let me check the weather for you.\"}]\n    }","title":"Examples - Hermes.Server.Response.assistant_message/2","ref":"Hermes.Server.Response.html#assistant_message/2-examples"},{"type":"function","doc":"Add audio content to a tool response.","title":"Hermes.Server.Response.audio/4","ref":"Hermes.Server.Response.html#audio/4"},{"type":"function","doc":"* `response` - A tool response struct\n  * `data` - Base64 encoded audio data\n  * `mime_type` - MIME type of the audio (e.g., \"audio/wav\")\n  * `opts` - Optional keyword list with:\n    * `:transcription` - Optional text transcription of the audio","title":"Parameters - Hermes.Server.Response.audio/4","ref":"Hermes.Server.Response.html#audio/4-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.audio(audio_data, \"audio/wav\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"audio\", \"data\" => audio_data, \"mimeType\" => \"audio/wav\"}],\n      isError: false\n    }\n    \n    iex> Response.tool() |> Response.audio(audio_data, \"audio/wav\", transcription: \"Hello\")\n    %Response{\n      type: :tool,\n      content: [%{\n        \"type\" => \"audio\",\n        \"data\" => audio_data,\n        \"mimeType\" => \"audio/wav\",\n        \"transcription\" => \"Hello\"\n      }],\n      isError: false\n    }","title":"Examples - Hermes.Server.Response.audio/4","ref":"Hermes.Server.Response.html#audio/4-examples"},{"type":"function","doc":"Set blob (base64) content for a resource response.","title":"Hermes.Server.Response.blob/2","ref":"Hermes.Server.Response.html#blob/2"},{"type":"function","doc":"* `response` - A resource response struct\n  * `data` - Base64 encoded binary data","title":"Parameters - Hermes.Server.Response.blob/2","ref":"Hermes.Server.Response.html#blob/2-parameters"},{"type":"function","doc":"iex> Response.resource() |> Response.blob(base64_data)\n    %Response{type: :resource, contents: %{\"blob\" => base64_data}}","title":"Examples - Hermes.Server.Response.blob/2","ref":"Hermes.Server.Response.html#blob/2-examples"},{"type":"function","doc":"Set optional description for a resource response.","title":"Hermes.Server.Response.description/2","ref":"Hermes.Server.Response.html#description/2"},{"type":"function","doc":"* `response` - A resource response struct\n  * `desc` - Description of the resource","title":"Parameters - Hermes.Server.Response.description/2","ref":"Hermes.Server.Response.html#description/2-parameters"},{"type":"function","doc":"iex> Response.resource() |> Response.description(\"Application configuration settings\")\n    %Response{type: :resource, metadata: %{description: \"Application configuration settings\"}}","title":"Examples - Hermes.Server.Response.description/2","ref":"Hermes.Server.Response.html#description/2-examples"},{"type":"function","doc":"Add an embedded resource reference to a tool response.","title":"Hermes.Server.Response.embedded_resource/3","ref":"Hermes.Server.Response.html#embedded_resource/3"},{"type":"function","doc":"* `response` - A tool response struct\n  * `uri` - The resource URI\n  * `opts` - Optional keyword list with:\n    * `:name` - Human-readable name\n    * `:description` - Resource description\n    * `:mime_type` - MIME type\n    * `:text` - Text content (for text resources)\n    * `:blob` - Base64 data (for binary resources)","title":"Parameters - Hermes.Server.Response.embedded_resource/3","ref":"Hermes.Server.Response.html#embedded_resource/3-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.embedded_resource(\"file://example.txt\",\n    ...>   name: \"Example File\",\n    ...>   mime_type: \"text/plain\",\n    ...>   text: \"File contents\"\n    ...> )","title":"Examples - Hermes.Server.Response.embedded_resource/3","ref":"Hermes.Server.Response.html#embedded_resource/3-examples"},{"type":"function","doc":"Mark a tool response as an error and add error message.","title":"Hermes.Server.Response.error/2","ref":"Hermes.Server.Response.html#error/2"},{"type":"function","doc":"* `response` - A tool response struct\n  * `message` - The error message","title":"Parameters - Hermes.Server.Response.error/2","ref":"Hermes.Server.Response.html#error/2-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.error(\"Division by zero\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"Error: Division by zero\"}],\n      isError: true\n    }","title":"Examples - Hermes.Server.Response.error/2","ref":"Hermes.Server.Response.html#error/2-examples"},{"type":"function","doc":"Add image content to a tool response.","title":"Hermes.Server.Response.image/3","ref":"Hermes.Server.Response.html#image/3"},{"type":"function","doc":"* `response` - A tool response struct\n  * `data` - Base64 encoded image data\n  * `mime_type` - MIME type of the image (e.g., \"image/png\")","title":"Parameters - Hermes.Server.Response.image/3","ref":"Hermes.Server.Response.html#image/3-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.image(base64_data, \"image/png\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"image\", \"data\" => base64_data, \"mimeType\" => \"image/png\"}],\n      isError: false\n    }","title":"Examples - Hermes.Server.Response.image/3","ref":"Hermes.Server.Response.html#image/3-examples"},{"type":"function","doc":"Add JSON-encoded content to a tool response.\n\nThis is a convenience function that automatically encodes data as JSON\nand adds it as text content. Useful for returning structured data from tools.","title":"Hermes.Server.Response.json/2","ref":"Hermes.Server.Response.html#json/2"},{"type":"function","doc":"* `response` - A tool response struct\n  * `data` - Any JSON-encodable data structure","title":"Parameters - Hermes.Server.Response.json/2","ref":"Hermes.Server.Response.html#json/2-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.json(%{status: \"ok\", count: 42})\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"{\\\"status\\\":\\\"ok\\\",\\\"count\\\":42}\"}],\n      isError: false\n    }\n    \n    iex> Response.tool() |> Response.json([1, 2, 3])\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"[1,2,3]\"}],\n      isError: false\n    }","title":"Examples - Hermes.Server.Response.json/2","ref":"Hermes.Server.Response.html#json/2-examples"},{"type":"function","doc":"Set optional name for a resource response.","title":"Hermes.Server.Response.name/2","ref":"Hermes.Server.Response.html#name/2"},{"type":"function","doc":"* `response` - A resource response struct\n  * `name` - Human-readable name for the resource","title":"Parameters - Hermes.Server.Response.name/2","ref":"Hermes.Server.Response.html#name/2-parameters"},{"type":"function","doc":"iex> Response.resource() |> Response.name(\"Configuration File\")\n    %Response{type: :resource, metadata: %{name: \"Configuration File\"}}","title":"Examples - Hermes.Server.Response.name/2","ref":"Hermes.Server.Response.html#name/2-examples"},{"type":"function","doc":"Start building a prompt response with optional description.","title":"Hermes.Server.Response.prompt/1","ref":"Hermes.Server.Response.html#prompt/1"},{"type":"function","doc":"* `description` - Optional description of the prompt","title":"Parameters - Hermes.Server.Response.prompt/1","ref":"Hermes.Server.Response.html#prompt/1-parameters"},{"type":"function","doc":"iex> Response.prompt()\n    %Response{type: :prompt, messages: []}\n    \n    iex> Response.prompt(\"Weather assistant prompt\")\n    %Response{type: :prompt, messages: [], description: \"Weather assistant prompt\"}","title":"Examples - Hermes.Server.Response.prompt/1","ref":"Hermes.Server.Response.html#prompt/1-examples"},{"type":"function","doc":"Start building a resource response.\n\nThe uri and mimeType are automatically injected from the component's\nuri/0 and mime_type/0 callbacks when the response is built by the server.","title":"Hermes.Server.Response.resource/0","ref":"Hermes.Server.Response.html#resource/0"},{"type":"function","doc":"iex> Response.resource()\n    %Response{type: :resource, contents: nil}","title":"Examples - Hermes.Server.Response.resource/0","ref":"Hermes.Server.Response.html#resource/0-examples"},{"type":"function","doc":"Add a system message to a prompt response.","title":"Hermes.Server.Response.system_message/2","ref":"Hermes.Server.Response.html#system_message/2"},{"type":"function","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","title":"Parameters - Hermes.Server.Response.system_message/2","ref":"Hermes.Server.Response.html#system_message/2-parameters"},{"type":"function","doc":"iex> Response.prompt() |> Response.system_message(\"You are a helpful weather assistant.\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"system\", \"content\" => \"You are a helpful weather assistant.\"}]\n    }","title":"Examples - Hermes.Server.Response.system_message/2","ref":"Hermes.Server.Response.html#system_message/2-examples"},{"type":"type","doc":"","title":"Hermes.Server.Response.t/0","ref":"Hermes.Server.Response.html#t:t/0"},{"type":"function","doc":"Add text content to a tool or resource response.\n\nFor tool responses, adds text to the content array.\nFor resource responses, sets the text content.","title":"Hermes.Server.Response.text/2","ref":"Hermes.Server.Response.html#text/2"},{"type":"function","doc":"* `response` - A tool or resource response struct\n  * `text` - The text content","title":"Parameters - Hermes.Server.Response.text/2","ref":"Hermes.Server.Response.html#text/2-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.text(\"Hello world\")\n    %Response{\n      type: :tool,\n      content: [%{\"type\" => \"text\", \"text\" => \"Hello world\"}],\n      isError: false\n    }\n    \n    iex> Response.resource() |> Response.text(\"File contents\")\n    %Response{type: :resource, contents: %{\"text\" => \"File contents\"}}","title":"Examples - Hermes.Server.Response.text/2","ref":"Hermes.Server.Response.html#text/2-examples"},{"type":"function","doc":"Build the final response structure.\n\nTransforms the response struct into the appropriate format for the MCP protocol.","title":"Hermes.Server.Response.to_protocol/1","ref":"Hermes.Server.Response.html#to_protocol/1"},{"type":"function","doc":"* `response` - A response struct of any type","title":"Parameters - Hermes.Server.Response.to_protocol/1","ref":"Hermes.Server.Response.html#to_protocol/1-parameters"},{"type":"function","doc":"iex> Response.tool() |> Response.text(\"Hello\") |> Response.to_protocol()\n    %{\"content\" => [%{\"type\" => \"text\", \"text\" => \"Hello\"}], \"isError\" => false}\n    \n    iex> Response.prompt() |> Response.user_message(\"Hi\") |> Response.to_protocol()\n    %{\"messages\" => [%{\"role\" => \"user\", \"content\" => \"Hi\"}]}\n    \n    iex> Response.resource() |> Response.text(\"data\") |> Response.to_protocol()\n    %{\"text\" => \"data\"}","title":"Examples - Hermes.Server.Response.to_protocol/1","ref":"Hermes.Server.Response.html#to_protocol/1-examples"},{"type":"function","doc":"","title":"Hermes.Server.Response.to_protocol/3","ref":"Hermes.Server.Response.html#to_protocol/3"},{"type":"function","doc":"Start building a tool response.","title":"Hermes.Server.Response.tool/0","ref":"Hermes.Server.Response.html#tool/0"},{"type":"function","doc":"iex> Response.tool()\n    %Response{type: :tool, content: [], isError: false}","title":"Examples - Hermes.Server.Response.tool/0","ref":"Hermes.Server.Response.html#tool/0-examples"},{"type":"function","doc":"Add a user message to a prompt response.","title":"Hermes.Server.Response.user_message/2","ref":"Hermes.Server.Response.html#user_message/2"},{"type":"function","doc":"* `response` - A prompt response struct\n  * `content` - The message content (string or structured content)","title":"Parameters - Hermes.Server.Response.user_message/2","ref":"Hermes.Server.Response.html#user_message/2-parameters"},{"type":"function","doc":"iex> Response.prompt() |> Response.user_message(\"What's the weather?\")\n    %Response{\n      type: :prompt,\n      messages: [%{\"role\" => \"user\", \"content\" => \"What's the weather?\"}]\n    }","title":"Examples - Hermes.Server.Response.user_message/2","ref":"Hermes.Server.Response.html#user_message/2-examples"},{"type":"module","doc":"SSE (Server-Sent Events) transport implementation for MCP servers.\n\nThis module provides backward compatibility with the HTTP+SSE transport\nfrom MCP protocol version 2024-11-05. It supports multiple concurrent\nclient sessions through Server-Sent Events for server-to-client communication\nand HTTP POST for client-to-server messages.","title":"Hermes.Server.Transport.SSE","ref":"Hermes.Server.Transport.SSE.html"},{"type":"module","doc":"- Multiple concurrent client sessions\n- Server-Sent Events for real-time server-to-client communication\n- Separate SSE and POST endpoints (as per 2024-11-05 spec)\n- Automatic session cleanup on disconnect\n- Integration with existing Phoenix/Plug applications","title":"Features - Hermes.Server.Transport.SSE","ref":"Hermes.Server.Transport.SSE.html#module-features"},{"type":"module","doc":"SSE transport is typically started through the server supervisor:\n\n    Hermes.Server.start_link(MyServer, [],\n      transport: :sse,\n      sse: [port: 8080, sse_path: \"/sse\", post_path: \"/messages\"]\n    )\n\nFor integration with existing Phoenix/Plug applications:\n\n    # In your router\n    forward \"/sse\", Hermes.Server.Transport.SSE.Plug,\n      server: MyApp.MCPServer,\n      mode: :sse\n\n    forward \"/messages\", Hermes.Server.Transport.SSE.Plug,\n      server: MyApp.MCPServer,\n      mode: :post","title":"Usage - Hermes.Server.Transport.SSE","ref":"Hermes.Server.Transport.SSE.html#module-usage"},{"type":"module","doc":"1. Client connects to SSE endpoint, receives \"endpoint\" event with POST URL\n2. Client sends messages via POST to the endpoint URL\n3. Server responses are pushed through the SSE connection\n4. Connection closes on client disconnect or server shutdown","title":"Message Flow - Hermes.Server.Transport.SSE","ref":"Hermes.Server.Transport.SSE.html#module-message-flow"},{"type":"module","doc":"- `:port` - HTTP server port (default: 8080)\n- `:sse_path` - Path for SSE connections (default: \"/sse\")\n- `:post_path` - Path for POST messages (default: \"/messages\")\n- `:server` - The MCP server process to connect to\n- `:name` - Process registration name","title":"Configuration - Hermes.Server.Transport.SSE","ref":"Hermes.Server.Transport.SSE.html#module-configuration"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Server.Transport.SSE.child_spec/1","ref":"Hermes.Server.Transport.SSE.html#child_spec/1"},{"type":"function","doc":"Gets the endpoint URL that should be sent to clients.\n\nThis constructs the URL that clients should use for POST requests.","title":"Hermes.Server.Transport.SSE.get_endpoint_url/1","ref":"Hermes.Server.Transport.SSE.html#get_endpoint_url/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.SSE.get_schema/1","ref":"Hermes.Server.Transport.SSE.html#get_schema/1"},{"type":"function","doc":"Gets the SSE handler process for a session.\n\nReturns the pid of the process handling SSE for this session,\nor nil if no SSE connection exists.","title":"Hermes.Server.Transport.SSE.get_sse_handler/2","ref":"Hermes.Server.Transport.SSE.html#get_sse_handler/2"},{"type":"function","doc":"Handles an incoming message from a client with request context.\n\nCalled by the Plug when a message is received via HTTP POST.","title":"Hermes.Server.Transport.SSE.handle_message/4","ref":"Hermes.Server.Transport.SSE.html#handle_message/4"},{"type":"type","doc":"SSE transport options\n\n- `:server` - The server process (required)\n- `:name` - Name for registering the GenServer (required)\n- `:base_url` - Base URL for constructing endpoint URLs\n- `:post_path` - Path for POST endpoint (default: \"/messages\")","title":"Hermes.Server.Transport.SSE.option/0","ref":"Hermes.Server.Transport.SSE.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Server.Transport.SSE.parse_options/1","ref":"Hermes.Server.Transport.SSE.html#parse_options/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.SSE.parse_options!/1","ref":"Hermes.Server.Transport.SSE.html#parse_options!/1"},{"type":"function","doc":"Registers an SSE handler process for a session.\n\nCalled by the Plug when establishing an SSE connection.\nThe calling process becomes the SSE handler for the session.","title":"Hermes.Server.Transport.SSE.register_sse_handler/2","ref":"Hermes.Server.Transport.SSE.html#register_sse_handler/2"},{"type":"function","doc":"Routes a message to a specific session's SSE handler.\n\nUsed for targeted server notifications to specific clients.","title":"Hermes.Server.Transport.SSE.route_to_session/3","ref":"Hermes.Server.Transport.SSE.html#route_to_session/3"},{"type":"function","doc":"Sends a message to the client via the active SSE connection.\n\nThis broadcasts to all active SSE connections for the session.","title":"Hermes.Server.Transport.SSE.send_message/2","ref":"Hermes.Server.Transport.SSE.html#send_message/2"},{"type":"function","doc":"* `transport` - The transport process\n  * `message` - The message to send","title":"Parameters - Hermes.Server.Transport.SSE.send_message/2","ref":"Hermes.Server.Transport.SSE.html#send_message/2-parameters"},{"type":"function","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","title":"Returns - Hermes.Server.Transport.SSE.send_message/2","ref":"Hermes.Server.Transport.SSE.html#send_message/2-returns"},{"type":"function","doc":"Shuts down the transport connection.\n\nThis terminates all active sessions managed by this transport.","title":"Hermes.Server.Transport.SSE.shutdown/1","ref":"Hermes.Server.Transport.SSE.html#shutdown/1"},{"type":"function","doc":"* `transport` - The transport process","title":"Parameters - Hermes.Server.Transport.SSE.shutdown/1","ref":"Hermes.Server.Transport.SSE.html#shutdown/1-parameters"},{"type":"function","doc":"Starts the SSE transport.","title":"Hermes.Server.Transport.SSE.start_link/1","ref":"Hermes.Server.Transport.SSE.html#start_link/1"},{"type":"type","doc":"","title":"Hermes.Server.Transport.SSE.t/0","ref":"Hermes.Server.Transport.SSE.html#t:t/0"},{"type":"function","doc":"Unregisters an SSE handler process for a session.\n\nCalled when the SSE connection is closed.","title":"Hermes.Server.Transport.SSE.unregister_sse_handler/2","ref":"Hermes.Server.Transport.SSE.html#unregister_sse_handler/2"},{"type":"module","doc":"A Plug implementation for the SSE (Server-Sent Events) transport.\n\nThis plug handles the MCP HTTP+SSE protocol as specified in MCP 2024-11-05.\nIt provides two separate endpoints:\n\n- SSE endpoint: Opens an SSE stream and sends \"endpoint\" event\n- POST endpoint: Handles JSON-RPC messages from client to server","title":"Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html"},{"type":"module","doc":"This Plug handles SSE streaming by keeping the request process alive \nand managing the streaming loop for server-to-client communication.","title":"SSE Streaming Architecture - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-sse-streaming-architecture"},{"type":"module","doc":"pipeline :mcp do\n      plug :accepts, [\"json\", \"event-stream\"]\n    end\n\n    scope \"/mcp\" do\n      pipe_through :mcp\n\n      # SSE endpoint\n      get \"/sse\", Hermes.Server.Transport.SSE.Plug,\n        server: :your_server_name, mode: :sse\n\n      # POST endpoint\n      post \"/messages\", Hermes.Server.Transport.SSE.Plug,\n        server: :your_server_name, mode: :post\n    end","title":"Usage in Phoenix Router - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-usage-in-phoenix-router"},{"type":"module","doc":"# When using in a standalone Plug.Router app\n    plug Hermes.Server.Transport.SSE.Plug,\n      server: :your_server_name,\n      mode: :sse,\n      at: \"/sse\",\n      method_whitelist: [\"GET\"]\n\n    plug Hermes.Server.Transport.SSE.Plug,\n      server: :your_server_name,\n      mode: :post,\n      at: \"/messages\",\n      method_whitelist: [\"POST\"]","title":"Usage in Plug Router (Standalone) - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-usage-in-plug-router-standalone"},{"type":"module","doc":"- `:server` - The server process name (required)\n- `:mode` - Either `:sse` or `:post` to determine endpoint behavior (required)\n- `:timeout` - Request timeout in milliseconds (default: 30000)\n- `:registry` - The registry to use. See `Hermes.Server.Registry.Adapter` for more information (default: Elixir's Registry implementation)","title":"Configuration Options - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-configuration-options"},{"type":"module","doc":"- Origin header validation for DNS rebinding protection\n- Session-based request validation\n- Automatic session cleanup on connection loss","title":"Security Features - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-security-features"},{"type":"module","doc":"- 200: Successful request or SSE stream established\n- 202: Accepted (for notifications)\n- 400: Bad request (malformed JSON-RPC)\n- 405: Method not allowed\n- 500: Internal server error","title":"HTTP Response Codes - Hermes.Server.Transport.SSE.Plug","ref":"Hermes.Server.Transport.SSE.Plug.html#module-http-response-codes"},{"type":"module","doc":"STDIO transport implementation for MCP servers.\n\nThis module handles communication with MCP clients via standard input/output streams,\nprocessing incoming JSON-RPC messages and forwarding responses.","title":"Hermes.Server.Transport.STDIO","ref":"Hermes.Server.Transport.STDIO.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Server.Transport.STDIO.child_spec/1","ref":"Hermes.Server.Transport.STDIO.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.STDIO.get_schema/1","ref":"Hermes.Server.Transport.STDIO.html#get_schema/1"},{"type":"type","doc":"STDIO transport options\n\n- `:server` - The server process (required)\n- `:name` - Optional name for registering the GenServer","title":"Hermes.Server.Transport.STDIO.option/0","ref":"Hermes.Server.Transport.STDIO.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Server.Transport.STDIO.parse_options/1","ref":"Hermes.Server.Transport.STDIO.html#parse_options/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.STDIO.parse_options!/1","ref":"Hermes.Server.Transport.STDIO.html#parse_options!/1"},{"type":"function","doc":"Sends a message to the client via stdout.","title":"Hermes.Server.Transport.STDIO.send_message/2","ref":"Hermes.Server.Transport.STDIO.html#send_message/2"},{"type":"function","doc":"* `transport` - The transport process\n  * `message` - The message to send","title":"Parameters - Hermes.Server.Transport.STDIO.send_message/2","ref":"Hermes.Server.Transport.STDIO.html#send_message/2-parameters"},{"type":"function","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","title":"Returns - Hermes.Server.Transport.STDIO.send_message/2","ref":"Hermes.Server.Transport.STDIO.html#send_message/2-returns"},{"type":"function","doc":"Shuts down the transport connection.","title":"Hermes.Server.Transport.STDIO.shutdown/1","ref":"Hermes.Server.Transport.STDIO.html#shutdown/1"},{"type":"function","doc":"* `transport` - The transport process","title":"Parameters - Hermes.Server.Transport.STDIO.shutdown/1","ref":"Hermes.Server.Transport.STDIO.html#shutdown/1-parameters"},{"type":"function","doc":"Starts a new STDIO transport process.","title":"Hermes.Server.Transport.STDIO.start_link/1","ref":"Hermes.Server.Transport.STDIO.html#start_link/1"},{"type":"function","doc":"* `opts` - Options\n    * `:server` - (required) The server to forward messages to\n    * `:name` - Optional name for the GenServer process","title":"Parameters - Hermes.Server.Transport.STDIO.start_link/1","ref":"Hermes.Server.Transport.STDIO.html#start_link/1-parameters"},{"type":"function","doc":"iex> Hermes.Server.Transport.STDIO.start_link(server: my_server)\n    {:ok, pid}","title":"Examples - Hermes.Server.Transport.STDIO.start_link/1","ref":"Hermes.Server.Transport.STDIO.html#start_link/1-examples"},{"type":"type","doc":"","title":"Hermes.Server.Transport.STDIO.t/0","ref":"Hermes.Server.Transport.STDIO.html#t:t/0"},{"type":"module","doc":"StreamableHTTP transport implementation for MCP servers.\n\nThis module provides an HTTP-based transport layer that supports multiple\nconcurrent client sessions through Server-Sent Events (SSE). It enables\nweb-based MCP clients to communicate with the server using standard HTTP\nprotocols.","title":"Hermes.Server.Transport.StreamableHTTP","ref":"Hermes.Server.Transport.StreamableHTTP.html"},{"type":"module","doc":"- Multiple concurrent client sessions\n- Server-Sent Events for real-time server-to-client communication\n- HTTP POST endpoint for client-to-server messages\n- Automatic session cleanup on disconnect\n- Integration with Phoenix/Plug applications","title":"Features - Hermes.Server.Transport.StreamableHTTP","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-features"},{"type":"module","doc":"StreamableHTTP is typically started through the server supervisor:\n\n    Hermes.Server.start_link(MyServer, [],\n      transport: :streamable_http,\n      streamable_http: [port: 4000]\n    )\n\nFor integration with existing Phoenix/Plug applications:\n\n    # In your router\n    forward \"/mcp\", Hermes.Server.Transport.StreamableHTTP.Plug,\n      server: MyApp.MCPServer","title":"Usage - Hermes.Server.Transport.StreamableHTTP","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-usage"},{"type":"module","doc":"1. Client connects to `/sse` endpoint, receives a session ID\n2. Client sends messages via POST to `/messages` with session ID header\n3. Server responses are pushed through the SSE connection\n4. Connection closes on client disconnect or server shutdown","title":"Message Flow - Hermes.Server.Transport.StreamableHTTP","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-message-flow"},{"type":"module","doc":"- `:port` - HTTP server port (default: 4000)\n- `:server` - The MCP server process to connect to\n- `:name` - Process registration name","title":"Configuration - Hermes.Server.Transport.StreamableHTTP","ref":"Hermes.Server.Transport.StreamableHTTP.html#module-configuration"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Server.Transport.StreamableHTTP.child_spec/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.StreamableHTTP.get_schema/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#get_schema/1"},{"type":"function","doc":"Gets the SSE handler process for a session.\n\nReturns the pid of the process handling SSE for this session,\nor nil if no SSE connection exists.","title":"Hermes.Server.Transport.StreamableHTTP.get_sse_handler/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#get_sse_handler/2"},{"type":"function","doc":"Handles an incoming message from a client with request context.\n\nCalled by the Plug when a message is received via HTTP POST.","title":"Hermes.Server.Transport.StreamableHTTP.handle_message/4","ref":"Hermes.Server.Transport.StreamableHTTP.html#handle_message/4"},{"type":"function","doc":"Handles an incoming message with context and returns {:sse, response} if SSE handler exists.\n\nThis allows the Plug to know whether to stream the response via SSE\nor return it as a regular HTTP response.","title":"Hermes.Server.Transport.StreamableHTTP.handle_message_for_sse/4","ref":"Hermes.Server.Transport.StreamableHTTP.html#handle_message_for_sse/4"},{"type":"type","doc":"StreamableHTTP transport options\n\n- `:server` - The server process (required)\n- `:name` - Name for registering the GenServer (required)","title":"Hermes.Server.Transport.StreamableHTTP.option/0","ref":"Hermes.Server.Transport.StreamableHTTP.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Server.Transport.StreamableHTTP.parse_options/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#parse_options/1"},{"type":"function","doc":"","title":"Hermes.Server.Transport.StreamableHTTP.parse_options!/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#parse_options!/1"},{"type":"function","doc":"Registers an SSE handler process for a session.\n\nCalled by the Plug when establishing an SSE connection.\nThe calling process becomes the SSE handler for the session.","title":"Hermes.Server.Transport.StreamableHTTP.register_sse_handler/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#register_sse_handler/2"},{"type":"function","doc":"Routes a message to a specific session's SSE handler.\n\nUsed for targeted server notifications to specific clients.","title":"Hermes.Server.Transport.StreamableHTTP.route_to_session/3","ref":"Hermes.Server.Transport.StreamableHTTP.html#route_to_session/3"},{"type":"function","doc":"Sends a message to the client via the active SSE connection.\n\nThis function is used for server-initiated notifications.\nIt will broadcast to all active SSE connections.","title":"Hermes.Server.Transport.StreamableHTTP.send_message/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2"},{"type":"function","doc":"* `transport` - The transport process\n  * `message` - The message to send","title":"Parameters - Hermes.Server.Transport.StreamableHTTP.send_message/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2-parameters"},{"type":"function","doc":"* `:ok` if message was sent successfully\n  * `{:error, reason}` otherwise","title":"Returns - Hermes.Server.Transport.StreamableHTTP.send_message/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#send_message/2-returns"},{"type":"function","doc":"Shuts down the transport connection.\n\nThis terminates all active sessions managed by this transport.","title":"Hermes.Server.Transport.StreamableHTTP.shutdown/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#shutdown/1"},{"type":"function","doc":"* `transport` - The transport process","title":"Parameters - Hermes.Server.Transport.StreamableHTTP.shutdown/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#shutdown/1-parameters"},{"type":"function","doc":"Starts the StreamableHTTP transport.","title":"Hermes.Server.Transport.StreamableHTTP.start_link/1","ref":"Hermes.Server.Transport.StreamableHTTP.html#start_link/1"},{"type":"type","doc":"","title":"Hermes.Server.Transport.StreamableHTTP.t/0","ref":"Hermes.Server.Transport.StreamableHTTP.html#t:t/0"},{"type":"function","doc":"Unregisters an SSE handler process for a session.\n\nCalled when the SSE connection is closed.","title":"Hermes.Server.Transport.StreamableHTTP.unregister_sse_handler/2","ref":"Hermes.Server.Transport.StreamableHTTP.html#unregister_sse_handler/2"},{"type":"module","doc":"A Plug implementation for the Streamable HTTP transport.\n\nThis plug handles the MCP Streamable HTTP protocol as specified in MCP 2025-03-26.\nIt provides a single endpoint that supports both GET and POST methods:\n\n- GET: Opens an SSE stream for server-to-client communication\n- POST: Handles JSON-RPC messages from client to server\n- DELETE: Closes a session","title":"Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html"},{"type":"module","doc":"This Plug handles SSE streaming by keeping the request process alive \nand managing the streaming loop for server-to-client communication.","title":"SSE Streaming Architecture - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-sse-streaming-architecture"},{"type":"module","doc":"pipeline :mcp do\n      plug :accepts, [\"json\"]\n    end\n\n    scope \"/mcp\" do\n      pipe_through :mcp\n      forward \"/\", to: Hermes.Server.Transport.StreamableHTTP.Plug, server: :your_server_name\n    end","title":"Usage in Phoenix Router - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-usage-in-phoenix-router"},{"type":"module","doc":"forward \"/mcp\", to: Hermes.Server.Transport.StreamableHTTP.Plug, init_opts: [server: :your_server_name]","title":"Usage in Plug Router - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-usage-in-plug-router"},{"type":"module","doc":"- `:server` - The server process name (required)\n- `:session_header` - Custom header name for session ID (default: \"mcp-session-id\")\n- `:timeout` - Request timeout in milliseconds (default: 30000)\n- `:registry` - The registry to use. See `Hermes.Server.Registry.Adapter` for more information (default: Elixir's Registry implementation)","title":"Configuration Options - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-configuration-options"},{"type":"module","doc":"- Origin header validation for DNS rebinding protection\n- Session-based request validation\n- Automatic session cleanup on connection loss\n- Rate limiting support (when configured)","title":"Security Features - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-security-features"},{"type":"module","doc":"- 200: Successful request\n- 202: Accepted (for notifications and responses)\n- 400: Bad request (malformed JSON-RPC)\n- 404: Session not found\n- 405: Method not allowed\n- 500: Internal server error","title":"HTTP Response Codes - Hermes.Server.Transport.StreamableHTTP.Plug","ref":"Hermes.Server.Transport.StreamableHTTP.Plug.html#module-http-response-codes"},{"type":"behaviour","doc":"Defines the behavior that all transport implementations must follow.","title":"Hermes.Transport.Behaviour","ref":"Hermes.Transport.Behaviour.html"},{"type":"type","doc":"The JSON-RPC message encoded","title":"Hermes.Transport.Behaviour.message/0","ref":"Hermes.Transport.Behaviour.html#t:message/0"},{"type":"type","doc":"","title":"Hermes.Transport.Behaviour.reason/0","ref":"Hermes.Transport.Behaviour.html#t:reason/0"},{"type":"callback","doc":"","title":"Hermes.Transport.Behaviour.send_message/2","ref":"Hermes.Transport.Behaviour.html#c:send_message/2"},{"type":"callback","doc":"","title":"Hermes.Transport.Behaviour.shutdown/1","ref":"Hermes.Transport.Behaviour.html#c:shutdown/1"},{"type":"callback","doc":"","title":"Hermes.Transport.Behaviour.start_link/1","ref":"Hermes.Transport.Behaviour.html#c:start_link/1"},{"type":"callback","doc":"Returns the list of MCP protocol versions supported by this transport.","title":"Hermes.Transport.Behaviour.supported_protocol_versions/0","ref":"Hermes.Transport.Behaviour.html#c:supported_protocol_versions/0"},{"type":"callback","doc":"iex> MyTransport.supported_protocol_versions()\n    [\"2024-11-05\", \"2025-03-26\"]","title":"Examples - Hermes.Transport.Behaviour.supported_protocol_versions/0","ref":"Hermes.Transport.Behaviour.html#c:supported_protocol_versions/0-examples"},{"type":"type","doc":"","title":"Hermes.Transport.Behaviour.t/0","ref":"Hermes.Transport.Behaviour.html#t:t/0"},{"type":"module","doc":"A transport implementation that uses Server-Sent Events (SSE) for receiving messages\nand HTTP POST requests for sending messages back to the server.\n\n>","title":"Hermes.Transport.SSE","ref":"Hermes.Transport.SSE.html"},{"type":"module","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","title":"Notes - Hermes.Transport.SSE","ref":"Hermes.Transport.SSE.html#module-notes"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Transport.SSE.child_spec/1","ref":"Hermes.Transport.SSE.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Transport.SSE.get_schema/1","ref":"Hermes.Transport.SSE.html#get_schema/1"},{"type":"type","doc":"The options for the SSE transport.\n\n- `:name` - The name of the transport process, respecting the `GenServer` \"Name Registration\" section.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section.\n- `:server` - The server configuration.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying HTTP transport options to pass to the HTTP client. You can check on the [Mint docs](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options)\n- `:http_options` - The underlying HTTP client options to pass to the HTTP client. You can check on the [Finch docs](https://hexdocs.pm/finch/Finch.html#t:request_opt/0)","title":"Hermes.Transport.SSE.option/0","ref":"Hermes.Transport.SSE.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Transport.SSE.options_schema/1","ref":"Hermes.Transport.SSE.html#options_schema/1"},{"type":"function","doc":"","title":"Hermes.Transport.SSE.options_schema!/1","ref":"Hermes.Transport.SSE.html#options_schema!/1"},{"type":"type","doc":"","title":"Hermes.Transport.SSE.params_t/0","ref":"Hermes.Transport.SSE.html#t:params_t/0"},{"type":"type","doc":"The options for the MCP server.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:base_path` - The base path of the MCP server (e.g. /mcp).\n- `:sse_path` - The path to the SSE endpoint (e.g. /mcp/sse) (default `:base_path` + `/sse`).","title":"Hermes.Transport.SSE.server/0","ref":"Hermes.Transport.SSE.html#t:server/0"},{"type":"type","doc":"","title":"Hermes.Transport.SSE.t/0","ref":"Hermes.Transport.SSE.html#t:t/0"},{"type":"module","doc":"A transport implementation that uses standard input/output.\n\n>","title":"Hermes.Transport.STDIO","ref":"Hermes.Transport.STDIO.html"},{"type":"module","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","title":"Notes - Hermes.Transport.STDIO","ref":"Hermes.Transport.STDIO.html#module-notes"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Transport.STDIO.child_spec/1","ref":"Hermes.Transport.STDIO.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Transport.STDIO.get_schema/1","ref":"Hermes.Transport.STDIO.html#get_schema/1"},{"type":"type","doc":"The options for the STDIO transport.\n\n- `:command` - The command to run, it will be searched in the system's PATH.\n- `:args` - The arguments to pass to the command, as a list of strings.\n- `:env` - The extra environment variables to set for the command, as a map.\n- `:cwd` - The working directory for the command.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section\n\nAnd any other `GenServer` init option.","title":"Hermes.Transport.STDIO.option/0","ref":"Hermes.Transport.STDIO.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Transport.STDIO.options_schema/1","ref":"Hermes.Transport.STDIO.html#options_schema/1"},{"type":"function","doc":"","title":"Hermes.Transport.STDIO.options_schema!/1","ref":"Hermes.Transport.STDIO.html#options_schema!/1"},{"type":"type","doc":"","title":"Hermes.Transport.STDIO.params_t/0","ref":"Hermes.Transport.STDIO.html#t:params_t/0"},{"type":"type","doc":"","title":"Hermes.Transport.STDIO.t/0","ref":"Hermes.Transport.STDIO.html#t:t/0"},{"type":"module","doc":"A transport implementation that uses Streamable HTTP as specified in MCP 2025-03-26.\n\nThis transport communicates with MCP servers via HTTP POST requests for sending messages\nand optionally uses Server-Sent Events (SSE) for receiving streaming responses.","title":"Hermes.Transport.StreamableHTTP","ref":"Hermes.Transport.StreamableHTTP.html"},{"type":"module","doc":"# Start the transport with a base URL\n    {:ok, transport} = Hermes.Transport.StreamableHTTP.start_link(\n      client: client_pid,\n      base_url: \"http://localhost:8000\",\n      mcp_path: \"/mcp\"\n    )\n\n    # Send a message\n    :ok = Hermes.Transport.StreamableHTTP.send_message(transport, encoded_message)","title":"Usage - Hermes.Transport.StreamableHTTP","ref":"Hermes.Transport.StreamableHTTP.html#module-usage"},{"type":"module","doc":"The transport automatically handles MCP session IDs via the `mcp-session-id` header:\n- Extracts session ID from server responses\n- Includes session ID in subsequent requests\n- Maintains session state throughout the connection lifecycle\n- Handles session expiration (404 responses) by reinitializing","title":"Session Management - Hermes.Transport.StreamableHTTP","ref":"Hermes.Transport.StreamableHTTP.html#module-session-management"},{"type":"module","doc":"Based on the response status and content type:\n- 202 Accepted: Message acknowledged, no immediate response\n- 200 OK with application/json: Single JSON response forwarded to client\n- 200 OK with text/event-stream: SSE stream parsed and events forwarded to client\n- 404 Not Found: Session expired, triggers reinitialization","title":"Response Handling - Hermes.Transport.StreamableHTTP","ref":"Hermes.Transport.StreamableHTTP.html#module-response-handling"},{"type":"module","doc":"The transport can establish a separate GET connection for server-initiated messages.\nThis allows the server to send requests and notifications without a client request.","title":"SSE Support - Hermes.Transport.StreamableHTTP","ref":"Hermes.Transport.StreamableHTTP.html#module-sse-support"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Transport.StreamableHTTP.child_spec/1","ref":"Hermes.Transport.StreamableHTTP.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Transport.StreamableHTTP.get_schema/1","ref":"Hermes.Transport.StreamableHTTP.html#get_schema/1"},{"type":"type","doc":"The options for the Streamable HTTP transport.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:mcp_path` - The MCP endpoint path (e.g. /mcp) (default \"/mcp\").\n- `:client` - The client to send the messages to.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying HTTP transport options.\n- `:http_options` - The underlying HTTP client options.\n- `:enable_sse` - Whether to establish a GET connection for server-initiated messages (default false).","title":"Hermes.Transport.StreamableHTTP.option/0","ref":"Hermes.Transport.StreamableHTTP.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Transport.StreamableHTTP.options_schema/1","ref":"Hermes.Transport.StreamableHTTP.html#options_schema/1"},{"type":"function","doc":"","title":"Hermes.Transport.StreamableHTTP.options_schema!/1","ref":"Hermes.Transport.StreamableHTTP.html#options_schema!/1"},{"type":"type","doc":"","title":"Hermes.Transport.StreamableHTTP.params_t/0","ref":"Hermes.Transport.StreamableHTTP.html#t:params_t/0"},{"type":"type","doc":"","title":"Hermes.Transport.StreamableHTTP.t/0","ref":"Hermes.Transport.StreamableHTTP.html#t:t/0"},{"type":"module","doc":"A transport implementation that uses WebSockets for bidirectional communication\nwith the MCP server.\n\n>","title":"Hermes.Transport.WebSocket","ref":"Hermes.Transport.WebSocket.html"},{"type":"module","doc":">\n> For initialization and setup, check our [Installation & Setup](./installation.html) and\n> the [Transport options](./transport_options.html) guides for reference.","title":"Notes - Hermes.Transport.WebSocket","ref":"Hermes.Transport.WebSocket.html#module-notes"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Hermes.Transport.WebSocket.child_spec/1","ref":"Hermes.Transport.WebSocket.html#child_spec/1"},{"type":"function","doc":"","title":"Hermes.Transport.WebSocket.get_schema/1","ref":"Hermes.Transport.WebSocket.html#get_schema/1"},{"type":"type","doc":"The options for the WebSocket transport.\n\n- `:name` - The name of the transport process, respecting the `GenServer` \"Name Registration\" section.\n- `:client` - The client to send the messages to, respecting the `GenServer` \"Name Registration\" section.\n- `:server` - The server configuration.\n- `:headers` - The headers to send with the HTTP requests.\n- `:transport_opts` - The underlying transport options to pass to Gun.","title":"Hermes.Transport.WebSocket.option/0","ref":"Hermes.Transport.WebSocket.html#t:option/0"},{"type":"function","doc":"","title":"Hermes.Transport.WebSocket.options_schema/1","ref":"Hermes.Transport.WebSocket.html#options_schema/1"},{"type":"function","doc":"","title":"Hermes.Transport.WebSocket.options_schema!/1","ref":"Hermes.Transport.WebSocket.html#options_schema!/1"},{"type":"type","doc":"","title":"Hermes.Transport.WebSocket.params_t/0","ref":"Hermes.Transport.WebSocket.html#t:params_t/0"},{"type":"type","doc":"The options for the MCP server.\n\n- `:base_url` - The base URL of the MCP server (e.g. http://localhost:8000) (required).\n- `:base_path` - The base path of the MCP server (e.g. /mcp).\n- `:ws_path` - The path to the WebSocket endpoint (e.g. /mcp/ws) (default `:base_path` + `/ws`).","title":"Hermes.Transport.WebSocket.server/0","ref":"Hermes.Transport.WebSocket.html#t:server/0"},{"type":"type","doc":"","title":"Hermes.Transport.WebSocket.t/0","ref":"Hermes.Transport.WebSocket.html#t:t/0"},{"type":"task","doc":"Mix task to test the SSE transport implementation, interactively sending commands.","title":"mix hermes.sse.interactive","ref":"Mix.Tasks.Hermes.Sse.Interactive.html"},{"type":"task","doc":"* `--base-url` - Base URL for the SSE server (default: http://localhost:8000)\n* `--base-path` - Base path to append to the base URL\n* `--sse-path` - Specific SSE endpoint path\n* `--header` - Add a header to requests (can be specified multiple times)","title":"Options - mix hermes.sse.interactive","ref":"Mix.Tasks.Hermes.Sse.Interactive.html#module-options"},{"type":"task","doc":"mix hermes.sse.interactive --base-url http://localhost:8000\n    mix hermes.sse.interactive --header \"Authorization: Bearer token123\"\n    mix hermes.sse.interactive --header \"X-API-Key: secret\" --header \"X-Custom: value\"","title":"Examples - mix hermes.sse.interactive","ref":"Mix.Tasks.Hermes.Sse.Interactive.html#module-examples"},{"type":"function","doc":"Callback implementation for  Mix.Task.run/1 .","title":"Mix.Tasks.Hermes.Sse.Interactive.run/1","ref":"Mix.Tasks.Hermes.Sse.Interactive.html#run/1"},{"type":"task","doc":"Mix task to test the STDIO transport implementation, interactively sending commands.","title":"mix hermes.stdio.interactive","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html"},{"type":"task","doc":"* `--command` - Command to execute for the STDIO transport (default: \"mcp\")\n* `--args` - Comma-separated arguments for the command (default: \"run,priv/dev/echo/index.py\")","title":"Options - mix hermes.stdio.interactive","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html#module-options"},{"type":"function","doc":"Callback implementation for  Mix.Task.run/1 .","title":"Mix.Tasks.Hermes.Stdio.Interactive.run/1","ref":"Mix.Tasks.Hermes.Stdio.Interactive.html#run/1"},{"type":"task","doc":"Mix task to test the Streamable HTTP transport implementation, interactively sending commands.","title":"mix hermes.streamable_http.interactive","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html"},{"type":"task","doc":"* `--base-url` - Base URL for the MCP server (default: http://localhost:8000)\n* `--mcp-path` - MCP endpoint path (default: /mcp)\n* `--header` - Add a header to requests (can be specified multiple times)","title":"Options - mix hermes.streamable_http.interactive","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html#module-options"},{"type":"task","doc":"mix hermes.streamable_http.interactive --base-url http://localhost:8000\n    mix hermes.streamable_http.interactive --header \"Authorization: Bearer token123\"\n    mix hermes.streamable_http.interactive --header \"X-API-Key: secret\" --header \"X-Custom: value\"","title":"Examples - mix hermes.streamable_http.interactive","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html#module-examples"},{"type":"function","doc":"Callback implementation for  Mix.Task.run/1 .","title":"Mix.Tasks.Hermes.StreamableHttp.Interactive.run/1","ref":"Mix.Tasks.Hermes.StreamableHttp.Interactive.html#run/1"},{"type":"task","doc":"Mix task to test the WebSocket transport implementation, interactively sending commands.","title":"mix hermes.websocket.interactive","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html"},{"type":"task","doc":"* `--base-url` - Base URL for the WebSocket server (default: http://localhost:8000)\n* `--base-path` - Base path to append to the base URL\n* `--ws-path` - Specific WebSocket endpoint path (default: /ws)","title":"Options - mix hermes.websocket.interactive","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html#module-options"},{"type":"function","doc":"Callback implementation for  Mix.Task.run/1 .","title":"Mix.Tasks.Hermes.Websocket.Interactive.run/1","ref":"Mix.Tasks.Hermes.Websocket.Interactive.html#run/1"},{"type":"exception","doc":"Raised when required MCP server configuration is missing or invalid.\n\nThe MCP specification requires servers to provide:\n- `name`: A human-readable name for the server\n- `version`: The server's version string","title":"Hermes.Server.ConfigurationError","ref":"Hermes.Server.ConfigurationError.html"},{"type":"exception","doc":"# This will raise an error - missing required options\n    defmodule BadServer do\n      use Hermes.Server  # Raises Hermes.Server.ConfigurationError\n    end\n    \n    # This is correct\n    defmodule GoodServer do\n      use Hermes.Server,\n        name: \"My Server\",\n        version: \"1.0.0\"\n    end","title":"Examples - Hermes.Server.ConfigurationError","ref":"Hermes.Server.ConfigurationError.html#module-examples"},{"type":"extras","doc":"# Hermes MCP\n\n[![hex.pm](https://img.shields.io/hexpm/v/hermes_mcp.svg)](https://hex.pm/packages/hermes_mcp)\n[![docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/hermes_mcp)\n[![ci](https://github.com/cloudwalk/hermes-mcp/actions/workflows/ci.yml/badge.svg)](https://github.com/cloudwalk/hermes-mcp/actions/workflows/ci.yml)\n\nA high-performance Model Context Protocol (MCP) implementation in Elixir.","title":"Hermes MCP","ref":"readme.html"},{"type":"extras","doc":"Hermes MCP is a comprehensive Elixir SDK for the [Model Context Protocol](https://spec.modelcontextprotocol.io/), providing complete client and server implementations with Elixir's exceptional concurrency model and fault tolerance.","title":"Overview - Hermes MCP","ref":"readme.html#overview"},{"type":"extras","doc":"```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.11.3\"}  # x-release-please-version\n  ]\nend\n```","title":"Installation - Hermes MCP","ref":"readme.html#installation"},{"type":"extras","doc":"","title":"Quick Start - Hermes MCP","ref":"readme.html#quick-start"},{"type":"extras","doc":"```elixir\n# Define a server with tools capabilities\ndefmodule MyApp.MCPServer do\n  use Hermes.Server,\n    name: \"My Server\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  @impl true\n  # this callback will be called when the\n  # MCP initialize lifecycle completes\n  def init(_client_info, frame) do\n    {:ok,frame\n      |> assign(counter: 0)\n      |> register_tool(\"echo\",\n        input_schema: %{\n          text: {:required, :string, max: 150, description: \"the text to be echoed\"}\n        },\n        annotations: %{read_only: true},\n        description: \"echoes everything the user says to the LLM\") }\n  end\n\n  @impl true\n  def handle_tool(\"echo\", %{text: text}, frame) do\n    Logger.info(\"This tool was called #{frame.assigns.counter + 1}\")\n    {:reply, text, assign(frame, counter: frame.assigns.counter + 1)}\n  end\nend\n\n# Add to your application supervisor\nchildren = [\n  Hermes.Server.Registry,\n  {MyApp.MCPServer, transport: :streamable_http}\n]\n\n# Add to your Plug/Phoenix router (if using HTTP)\nforward \"/mcp\", Hermes.Server.Transport.StreamableHTTP.Plug, server: MyApp.MCPServer\n```\n\nNow you can achieve your MCP server on `http://localhost: /mcp`","title":"Server - Hermes MCP","ref":"readme.html#server"},{"type":"extras","doc":"```elixir\n# Define a client module\ndefmodule MyApp.MCPClient do\n  use Hermes.Client,\n    name: \"MyApp\",\n    version: \"1.0.0\",\n    protocol_version: \"2025-03-26\"\nend\n\n# Add to your application supervisor\nchildren = [\n  {MyApp.MCPClient,\n   transport: {:streamable_http, base_url: \"http://localhost:4000\"}}\n]\n\n# Use the client\n{:ok, result} = MyApp.MCPClient.call_tool(\"echo\", %{text: \"this will be echoed!\"})\n```","title":"Client - Hermes MCP","ref":"readme.html#client"},{"type":"extras","doc":"Named after Hermes, the Greek god of boundaries and communication, this library facilitates seamless interaction between Large Language Models and external tools - serving as a messenger between AI and data sources.","title":"Why Hermes? - Hermes MCP","ref":"readme.html#why-hermes"},{"type":"extras","doc":"For detailed guides and examples, visit the [official documentation](https://hexdocs.pm/hermes_mcp).","title":"Documentation - Hermes MCP","ref":"readme.html#documentation"},{"type":"extras","doc":"We have build some elixir implementation examples using `plug` based and `phoenix` apps:\n\n1. [upcase-server](/priv/dev/upcase/README.md): `plug` based MCP server using streamable_http\n2. [echo-elixir](/priv/dev/echo-elixir/README.md): `phoenix` based MCP server using sse\n3. [ascii-server](/priv/dev/ascii/README.md): `phoenix_live_view` based MCP server using streamable_http and UI","title":"Examples - Hermes MCP","ref":"readme.html#examples"},{"type":"extras","doc":"MIT License. See [LICENSE](./LICENSE) for details.","title":"License - Hermes MCP","ref":"readme.html#license"},{"type":"extras","doc":"# Welcome to Hermes MCP\n\nLet's explore how Hermes helps you build bridges between AI assistants and your Elixir applications. Have you ever wished your AI could directly interact with your code? That's exactly what we're enabling here.","title":"Welcome to Hermes MCP","ref":"home.html"},{"type":"extras","doc":"Hermes implements the Model Context Protocol (MCP) - think of it as a secure conversation channel between AI assistants and your applications. Your AI can discover what your app offers and interact with it safely.\n\nThree minutes. That's all we need to show you something working.","title":"What's This All About? - Welcome to Hermes MCP","ref":"home.html#what-s-this-all-about"},{"type":"extras","doc":"What if we started by connecting to an existing MCP server? Let's see how this feels:\n\n```elixir\n# In your mix.exs\n{:hermes_mcp, \"~> 0.11.2\"} # x-release-please-version\n```\n\nNow, let's define a client that can talk to Claude's own MCP server:\n\n```elixir\ndefmodule MyApp.ClaudeClient do\n  use Hermes.Client,\n    name: \"MyApp\",\n    version: \"1.0.0\",\n    protocol_version: \"2024-11-05\",\n    capabilities: [:roots]\nend\n```\n\nAdd it to your application:\n\n```elixir\n# considering you have `claude-code` installed\nchildren = [\n  {MyApp.ClaudeClient,\n   transport: {:stdio, command: \"claude\", args: [\"mcp\", \"serve\"]}}\n]\n```\n\nWhat can your client do now? Let's explore:\n\n```elixir\n# Is anyone there?\nMyApp.ClaudeClient.ping()\n# => :pong\n\n# What tools are available?\n{:ok, response} = MyApp.ClaudeClient.list_tools()\n# => Discover web search, news, and more...\n\n# Let's search for something\n{:ok, result} = MyApp.ClaudeClient.call_tool(\"search\", %{query: \"elixir lang\"})\n```\n\nHow does that feel? In just a few lines, you've connected your Elixir app to Claude's capabilities.","title":"Your First Connection - Welcome to Hermes MCP","ref":"home.html#your-first-connection"},{"type":"extras","doc":"What if you want to expose your own functionality to AI assistants? Let's create a simple server:\n\n```elixir\ndefmodule MyApp.Server do\n  use Hermes.Server,\n    name: \"my-app\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  component MyApp.Greeter\nend\n\ndefmodule MyApp.Greeter do\n  @moduledoc \"Greet someone warmly\"\n\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :name, :string, required: true\n  end\n\n  def execute(%{name: name}, _frame) do\n    {:ok, \"Hello #{name}! Welcome to the MCP world!\"}\n  end\nend\n\n# on your application.ex\nchildren = [\n  Hermes.Server.Registry,\n  {MyApp.Greeter, transport: :stdio}\n]\n```\n\nStart your server and any AI assistant can now discover and use your greeter. Interesting, right?","title":"Building Your Own Server - Welcome to Hermes MCP","ref":"home.html#building-your-own-server"},{"type":"extras","doc":"What interests you most?\n\n- **[Building a Client](building-a-client.md)** - Connect to any MCP server and leverage its capabilities\n- **[Building a Server](building-a-server.md)** - Expose your Elixir application's features to AI assistants\n- **[Recipes](recipes.md)** - Real-world patterns we've discovered along the way\n\nThe protocol handles all the complexity - transport negotiation, capability exchange, error handling. You just focus on what your application does best.\n\nReady to explore further?","title":"Where to Go Next? - Welcome to Hermes MCP","ref":"home.html#where-to-go-next"},{"type":"extras","doc":"# Building a Client\n\nLet's explore how to connect your Elixir application to MCP servers. What possibilities open up when your code can leverage AI-enhanced services?","title":"Building a Client","ref":"building-a-client.html"},{"type":"extras","doc":"Remember our first client? Let's understand what's happening:\n\n```elixir\ndefmodule MyApp.WeatherClient do\n  use Hermes.Client,\n    name: \"MyApp\",                    # How you introduce yourself\n    version: \"1.0.0\",                 # Your client's version\n    protocol_version: \"2024-11-05\",   # MCP protocol target version\n    capabilities: [:roots]            # What features you support\nend\n```\n\nWhen you add this to your supervision tree, something interesting happens:\n\n```elixir\n{MyApp.WeatherClient,\n transport: {:stdio, command: \"weather-server\", args: []}}\n```\n\nThe client automatically:\n\n- Launches the weather server as a subprocess\n- Negotiates capabilities\n- Maintains the connection\n- Handles all the protocol details\n\nHow might you use this connection?","title":"Starting Simple - Building a Client","ref":"building-a-client.html#starting-simple"},{"type":"extras","doc":"What can a connected server actually do? Let's find out:\n\n```elixir\n# What's this server about?\ninfo = MyApp.WeatherClient.get_server_info()\n# => %{\"name\" => \"Weather Server\", \"version\" => \"2.0.0\", ...}\n\n# What capabilities does it offer?\ncaps = MyApp.WeatherClient.get_server_capabilities()\n# => %{\"tools\" => %{\"listChanged\" => false}, ...}\n\n# What tools are available?\n{:ok, %{result: %{\"tools\" => tools}}} = MyApp.WeatherClient.list_tools()\n\nEnum.each(tools, fn tool ->\n  IO.puts(\"#{tool[\"name\"]}: #{tool[\"description\"]}\")\nend)\n# => get_weather: Get current weather for a location\n# => get_forecast: Get weather forecast\n```\n\nNotice how we're exploring the server's interface dynamically?","title":"Discovering Capabilities - Building a Client","ref":"building-a-client.html#discovering-capabilities"},{"type":"extras","doc":"Now for the interesting part - actually using these discovered tools:\n\n```elixir\n# Simple tool call\n{:ok, %{result: weather}} =\n  MyApp.WeatherClient.call_tool(\"get_weather\", %{\n    \"location\" => \"San Francisco\"\n  })\n\n# Tool with complex parameters\n{:ok, %{result: forecast}} =\n  MyApp.WeatherClient.call_tool(\"get_forecast\", %{\n    \"location\" => \"Tokyo\",\n    \"days\" => 5,\n    \"units\" => \"metric\"\n  })\n```\n\nWhat happens if something goes wrong?\n\n```elixir\ncase MyApp.WeatherClient.call_tool(\"get_weather\", %{\"location\" => \"\"}) do\n  {:ok, %{is_error: false, result: weather}} ->\n    # Success path\n\n  {:ok, %{is_error: true, result: error}} ->\n    # The tool itself reported an error\n    IO.puts(\"Tool error: #{error[\"message\"]}\")\n\n  {:error, error} ->\n    # Protocol or connection error\n    IO.puts(\"Connection error: #{inspect(error)}\")\nend\n```","title":"Using Tools - Building a Client","ref":"building-a-client.html#using-tools"},{"type":"extras","doc":"Some servers expose resources - think files, databases, or any readable content:\n\n```elixir\n# What resources are available?\n{:ok, %{result: %{\"resources\" => resources}}} =\n  MyApp.WeatherClient.list_resources()\n\n# Read a specific resource\n{:ok, %{result: %{\"contents\" => contents}}} =\n  MyApp.WeatherClient.read_resource(\"weather://stations/KSFO\")\n\n# Resources can have multiple content types\nfor content <- contents do\n  case content do\n    %{\"text\" => text} ->\n      IO.puts(\"Text content: #{text}\")\n\n    %{\"blob\" => blob} ->\n      IO.puts(\"Binary data: #{byte_size(blob)} bytes\")\n  end\nend\n```","title":"Working with Resources - Building a Client","ref":"building-a-client.html#working-with-resources"},{"type":"extras","doc":"How does your client actually connect to servers? Let's explore the options:\n\n```elixir\n# Local subprocess\ntransport: {:stdio, command: \"python\", args: [\"-m\", \"my_server\"]}\n\n# HTTP endpoint\ntransport: {:streamable_http, base_url: \"http://localhost:8000\"}\n\n# WebSocket for real-time\ntransport: {:websocket, base_url: \"ws://localhost:8000\"}\n\n# Server-Sent Events\ntransport: {:sse, base_url: \"http://localhost:8000\"}\n```\n\nWhich transport should you choose?\n\n- **STDIO**: Perfect for local tools and subprocess isolation\n- **HTTP**: Great for remote services and web APIs\n- **WebSocket**: When you need bidirectional real-time communication\n- **SSE**: For servers that push updates to clients (deprecated)","title":"Transport Options - Building a Client","ref":"building-a-client.html#transport-options"},{"type":"extras","doc":"","title":"Advanced Patterns - Building a Client","ref":"building-a-client.html#advanced-patterns"},{"type":"extras","doc":"Need to connect to multiple servers? No problem:\n\n```elixir\nchildren = [\n  Supervisor.child_spec(\n    {MyApp.WeatherClient,\n     name: :weather_us,\n     transport: {:stdio, command: \"weather-server\", args: [\"--region\", \"US\"]}},\n    id: :weather_us\n  ),\n\n  Supervisor.child_spec(\n    {MyApp.WeatherClient,\n     name: :weather_eu,\n     transport: {:stdio, command: \"weather-server\", args: [\"--region\", \"EU\"]}},\n    id: :weather_eu\n  )\n]\n\n# Use specific instances\nMyApp.WeatherClient.call_tool(:weather_us, \"get_weather\", %{location: \"NYC\"})\nMyApp.WeatherClient.call_tool(:weather_eu, \"get_weather\", %{location: \"Paris\"})\n```","title":"Multiple Client Instances - Building a Client","ref":"building-a-client.html#multiple-client-instances"},{"type":"extras","doc":"Long-running operations? Adjust timeouts:\n\n```elixir\n# 5 minute timeout for slow operations\nopts = [timeout: 300_000]\nMyApp.WeatherClient.call_tool(\"analyze_historical_data\", params, opts)\n```","title":"Handling Timeouts - Building a Client","ref":"building-a-client.html#handling-timeouts"},{"type":"extras","doc":"Need to track progress on long-running operations? Here's how:\n\n```elixir\n# Generate a unique token for this operation\nprogress_token = Hermes.MCP.ID.generate_progress_token()\n\n# Option 1: Just track with a token\nMyApp.WeatherClient.call_tool(\"analyze_data\", params,\n  progress: [token: progress_token]\n)\n\n# Option 2: Receive real-time updates\ncallback = fn ^progress_token, progress, total ->\n  percentage = if total, do: \"#{progress}/#{total}\", else: \"#{progress}\"\n  IO.puts(\"Progress: #{percentage}\")\nend\n\nMyApp.WeatherClient.call_tool(\"analyze_data\", params,\n  progress: [token: progress_token, callback: callback]\n)\n```\n\nThe server sends progress notifications that your callback receives automatically.","title":"Progress Tracking - Building a Client","ref":"building-a-client.html#progress-tracking"},{"type":"extras","doc":"When you're done:\n\n```elixir\nMyApp.WeatherClient.close()\n```\n\nThis cleanly shuts down the connection and any associated resources.","title":"Graceful Shutdown - Building a Client","ref":"building-a-client.html#graceful-shutdown"},{"type":"extras","doc":"Now that you understand clients, what interests you?\n\n- Building your own server to expose functionality?\n- Exploring specific recipes for common patterns?\n- Understanding how to handle errors gracefully?\n\nThe client abstraction handles all the protocol complexity - you just focus on using the capabilities. What will you connect to first?","title":"What's Next? - Building a Client","ref":"building-a-client.html#what-s-next"},{"type":"extras","doc":"# Building a Server\n\nWhat if your Elixir application could become a capability that AI assistants can discover and use? Let's explore how to expose your application's features through MCP.","title":"Building a Server","ref":"building-a-server.html"},{"type":"extras","doc":"Remember our greeter from the introduction? Let's understand what's really happening:\n\n```elixir\ndefmodule MyApp.Greeter do\n  @moduledoc \"Greet someone warmly\"\n\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :name, :string, required: true\n  end\n\n  def execute(%{name: name}, _frame) do\n    {:ok, \"Hello #{name}! Welcome to the MCP world!\"}\n  end\nend\n```\n\nWhat makes this special? When an AI assistant connects to your server, it can:\n\n- Discover this tool exists\n- Understand what parameters it needs\n- Call it with the right data\n- Get a response back\n\nThe `schema` block defines what the tool expects. The `execute` function does the work. That's it.","title":"Your First Tool - Building a Server","ref":"building-a-server.html#your-first-tool"},{"type":"extras","doc":"Now let's build a server that exposes this tool:\n\n```elixir\ndefmodule MyApp.Server do\n  use Hermes.Server,\n    name: \"my-app\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  # Register our greeter tool\n  component MyApp.Greeter\nend\n```\n\nAdd it to your supervision tree:\n\n```elixir\nchildren = [\n  # Start with STDIO for easy testing\n  {MyApp.Server, transport: :stdio}\n]\n```\n\nHow do you test this? Complete one file for reference:\n\n```elixir\nMix.install([{:hermes_mcp, \"~> 0.11\"}])\n\ndefmodule MyApp.Greeter do\n  @moduledoc \"Greet someone warmly\"\n\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :name, :string, required: true\n  end\n\n  def execute(%{name: name}, _frame) do\n    {:ok, \"Hello #{name}! Welcome to the MCP world!\"}\n  end\nend\n\ndefmodule MyApp.Server do\n  use Hermes.Server,\n    name: \"my-app\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  # Register our greeter tool\n  component MyApp.Greeter\nend\n\nchildren = [Hermes.Server.Registry, {MyApp.Server, transport: :stdio}]\n{:ok, _pid} = Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n```\n\nSave it to `my_app.exs` and you can test it with the helper task:\n\n```bash\nmix hermes.stdio.interactive -c elixir --args=--no-halt,my_app.exs\n```\n\nOr you can add it to claude, assuming you have `claude-code` installed:\n\n```bash\nclaude mcp add my-app -- elixir --no-halt my_app.exs\n```","title":"Creating Your Server - Building a Server","ref":"building-a-server.html#creating-your-server"},{"type":"extras","doc":"Let's create something more substantial. What if we built a tool that searches through your application's data?\n\n```elixir\ndefmodule MyApp.ProductSearch do\n  @moduledoc \"Search for products in our catalog\"\n\n  use Hermes.Server.Component, type: :tool\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :query, :string, required: true\n    field :limit, :integer, default: 10\n    field :category, :string\n  end\n\n  @impl true\n  def execute(%{query: query} = params, frame) do\n    limit = params[:limit] || 10\n    category = params[:category]\n\n    products =\n      MyApp.Catalog.search(query)\n      |> maybe_filter_by_category(category)\n      |> Enum.take(limit)\n      |> Enum.map(&format_product/1)\n\n    {:reply, Response.json(Response.tool(), products), frame}\n  end\n\n  defp maybe_filter_by_category(products, nil), do: products\n  defp maybe_filter_by_category(products, category) do\n    Enum.filter(products, &(&1.category == category))\n  end\n\n  defp format_product(product) do\n    %{\n      id: product.id,\n      name: product.name,\n      price: product.price,\n      description: product.description\n    }\n  end\nend\n```\n\nNotice how we're using your existing business logic? The tool is just a thin wrapper that makes it accessible to AI.","title":"Building Real Tools - Building a Server","ref":"building-a-server.html#building-real-tools"},{"type":"extras","doc":"Tools perform actions. Resources provide data. What if AI assistants could read your application's data directly?\n\n```elixir\ndefmodule MyApp.ConfigResource do\n  @moduledoc \"Current application configuration\"\n\n  use Hermes.Server.Component,\n    type: :resource,\n    uri: \"config://app/settings\"\n\n  alias Hermes.Server.Response\n\n  @impl true\n  def read(_params, frame) do\n    config = %{\n      environment: Application.get_env(:my_app, :environment),\n      features: Application.get_env(:my_app, :feature_flags),\n      version: Application.spec(:my_app, :vsn) |> to_string()\n    }\n\n    {:reply, Response.json(Response.resource(), config), frame}\n  end\nend\n```\n\nResources have URIs. AI assistants can discover and read them:\n\n```\nClient: list_resources()\nServer: [{uri: \"config://app/settings\", name: \"Application Config\", ...}]\nClient: read_resource(\"config://app/settings\")\nServer: {contents: [{text: \"{\\n  \\\"environment\\\": \\\"production\\\",\\n  ...\"}]}\n```","title":"Adding Resources - Building a Server","ref":"building-a-server.html#adding-resources"},{"type":"extras","doc":"Prompts are templates that help AI assistants interact with your users more effectively:\n\n```elixir\ndefmodule MyApp.BugReportPrompt do\n  @moduledoc \"Generate a structured bug report\"\n\n  use Hermes.Server.Component, type: :prompt\n\n  alias Hermes.Server.Response\n\n  schema do\n    field :title, :string, required: true\n    field :severity, :string, values: [\"low\", \"medium\", \"high\", \"critical\"]\n    field :steps_to_reproduce, :string\n    field :expected_behavior, :string\n    field :actual_behavior, :string\n  end\n\n  @impl true\n  def get_messages(params, frame) do\n    content = build_report_content(params)\n\n    response =\n      Response.prompt()\n      |> Response.user_message(content)\n      |> Response.system_message(\"This is the Bug report prompt, user already have the data\")\n\n    {:reply, response, frame}\n  end\n\n  defp bug_report_content(params) do\n    \"\"\"\n    Please help me file a bug report for: #{params.title}\n\n    Severity: #{params.severity || \"not specified\"}\n\n    Steps to reproduce:\n    #{params.steps_to_reproduce || \"not provided\"}\n\n    Expected behavior:\n    #{params.expected_behavior || \"not provided\"}\n\n    Actual behavior:\n    #{params.actual_behavior || \"not provided\"}\n\n    Please format this as a proper bug report and suggest any missing information.\n    \"\"\"\n  end\nend\n```","title":"Creating Prompts - Building a Server","ref":"building-a-server.html#creating-prompts"},{"type":"extras","doc":"How do clients connect to your server? Let's explore your options:","title":"Transport Options - Building a Server","ref":"building-a-server.html#transport-options"},{"type":"extras","doc":"Perfect for CLI tools and development:\n\n```elixir\n{MyApp.Server, transport: :stdio}\n```\n\nYour server communicates through standard input/output. Great for:\n\n- Command-line tools\n- Development and testing\n- Subprocess isolation","title":"STDIO (Development & CLIs) - Building a Server","ref":"building-a-server.html#stdio-development-clis"},{"type":"extras","doc":"For web services that multiple clients connect to:\n\n```elixir\n{MyApp.Server, transport: {:streamable_http, port: 8080}}\n```\n\nThis creates an HTTP endpoint at `http://localhost:8080/mcp`. Each client gets its own session.","title":"HTTP (Web Applications) - Building a Server","ref":"building-a-server.html#http-web-applications"},{"type":"extras","doc":"Already have a Phoenix app? Integrate MCP as a route:\n\n```elixir\n# In your Phoenix endpoint (lib/my_app_web/endpoint.ex)\ndefmodule MyAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_app\n\n  # Add the MCP plug before your router\n  plug Hermes.Server.Transport.StreamableHTTP.Plug,\n    server: MyApp.Server,\n    path: \"/mcp\"\n\n  # Your other plugs...\n  plug MyAppWeb.Router\nend\n\n# In your application supervisor\nchildren = [\n  MyAppWeb.Endpoint,\n  Hermes.Server.Registry,\n  {MyApp.Server, transport: :streamable_http}\n]\n```\n\nNow your MCP server is available at `http://localhost:4000/mcp`.","title":"Integration with Phoenix - Building a Server","ref":"building-a-server.html#integration-with-phoenix"},{"type":"extras","doc":"What happens when things go wrong? Let's handle errors gracefully:\n\n```elixir\ndefmodule MyApp.DatabaseQuery do\n  @moduledoc \"Query the database\"\n\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :query, :string, required: true\n  end\n\n  @impl true\n  def execute(%{query: query}, frame) do\n    case MyApp.Repo.query(query) do\n      {:ok, result} ->\n        {:reply, Response.json(Response.tool(), format_result(result)), frame}\n\n      {:error, reason} ->\n        {:reply, Response.error(Response.tool(), \"Query failed: #{to_string(reason)}\")}\n    end\n  end\nend\n```\n\nHermes automatically formats your error responses according to the MCP protocol.","title":"Error Handling - Building a Server","ref":"building-a-server.html#error-handling"},{"type":"extras","doc":"Need to maintain state across calls? The frame provides context:\n\n```elixir\ndefmodule MyApp.Conversation do\n  @moduledoc \"Continue a conversation\"\n\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :message, :string, required: true\n  end\n\n  @impl true\n  def execute(%{message: message}, frame) do\n    session_id = frame.private.session_id\n    history = ConversationStore.get_history(session_id)\n\n    new_history = history ++ [message]\n    ConversationStore.save_history(session_id, new_history)\n\n    content = generate_response(new_history)\n\n    {:reply, Response.text(Response.tool(), content), frame}\n  end\nend\n```","title":"Stateful Operations - Building a Server","ref":"building-a-server.html#stateful-operations"},{"type":"extras","doc":"Need to add extra metadata to your tools? Annotations provide additional context:\n\n```elixir\ndefmodule MyApp.DatabaseQuery do\n  @moduledoc \"Query the application database\"\n\n  use Hermes.Server.Component,\n    type: :tool,\n    annotations: %{\n      \"x-api-version\" => \"2.0\",\n      \"x-rate-limit\" => \"10/minute\",\n      \"x-auth-required\" => true\n    }\n\n  schema do\n    field :query, :string, required: true\n  end\n\n  @impl true\n  def execute(params, frame) do\n    # Implementation...\n  end\nend\n```\n\nThese annotations are exposed in the tool definition, helping clients understand additional constraints or requirements.","title":"Tool Annotations - Building a Server","ref":"building-a-server.html#tool-annotations"},{"type":"extras","doc":"How do you know your server works correctly? Let's explore interactive testing first:","title":"Testing Your Server - Building a Server","ref":"building-a-server.html#testing-your-server"},{"type":"extras","doc":"Hermes provides interactive `Mix` tasks for different transports if you need quick testing:\n\n```bash\n# Test STDIO server\nmix hermes.stdio.interactive --command elixir --args=--no-halt,my_app.exs\n\n# Test HTTP server\nmix hermes.streamable_http.interactive --base-url=http://localhost:8080 --header 'authorization: Bearer 123'\n\n# With verbose logging\nmix hermes.stdio.sse --base-url=http//:localhost:4000 -vvv\n```\n\nIn the interactive session:\n\n```\nmcp> ping\npong\n\nmcp> list_tools\nAvailable tools:\n- greeter: Greet someone warmly\n- product_search: Search for products in our catalog\n\nmcp> call_tool\nTool name: greeter\nTool arguments (JSON): {\"name\": \"Alice\"}\nResult: Hello Alice! Welcome to the MCP world!\n\nmcp> show_state\nClient State:\n  Protocol: 2024-11-05\n  Initialized: true\n  ...\n```","title":"Interactive CLI Testing - Building a Server","ref":"building-a-server.html#interactive-cli-testing"},{"type":"extras","doc":"Now let's write some tests:\n\n```elixir\ndefmodule MyApp.ServerTest do\n  use ExUnit.Case\n\n  alias Hermes.Server.Frame\n\n  test \"greeter tool works correctly\" do\n    frame = %Frame{}\n\n    assert {:reply, resp, ^frame} = Greeter.execute(%{name: \"joe\"}, frame)\n    assert {:ok, %{\"result\" => %{\"content\" => content}}} = JSON.decode(resp)\n    assert [%{\"text\" => \"Hello joe! Welcome to the MCP world!\"}] = content\n  end\nend\n```","title":"Unit Testing - Building a Server","ref":"building-a-server.html#unit-testing"},{"type":"extras","doc":"You've seen how to expose your Elixir application's capabilities to AI assistants. What patterns interest you most?\n\n- Complex multi-step workflows?\n- Authentication and authorization?\n- Real-time updates and notifications?\n\nThe server abstraction handles all the protocol complexity. You just focus on what your application does best.","title":"What's Next? - Building a Server","ref":"building-a-server.html#what-s-next"},{"type":"extras","doc":"# Recipes\n\nLet's explore patterns we've discovered while building with Hermes MCP. What challenges are you facing?","title":"Recipes","ref":"recipes.html"},{"type":"extras","doc":"How do you ensure only authorized clients can access your server? Let's explore a few approaches:","title":"Authentication & Authorization - Recipes","ref":"recipes.html#authentication-authorization"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.AuthenticatedServer do\n  use Hermes.Server,\n    name: \"secure-app\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  def init(arg, frame) do\n    # Check API key from transport metadata\n    api_key = get_in(frame.transport, [:headers, \"x-api-key\"])\n\n    case authenticate_api_key(api_key) do\n      {:ok, user} ->\n        # Store user in frame assigns for later use\n        {:ok, Map.put(frame, :assigns, %{user: user})}\n\n      :error ->\n        {:stop, :unauthorized}\n    end\n  end\n\n  defp authenticate_api_key(nil), do: :error\n  defp authenticate_api_key(key) do\n    # Your authentication logic here\n    MyApp.Auth.verify_api_key(key)\n  end\nend\n```\n\nNow your tools can access the authenticated user:\n\n```elixir\ndefmodule MyApp.SecureTool do\n  use Hermes.Server.Component, type: :tool\n\n  def execute(params, frame) do\n    user = frame.assigns.user\n\n    # User-scoped operations\n    {:ok, \"Hello #{user.name}, you have access to this tool!\"}\n  end\nend\n```","title":"API Key Authentication - Recipes","ref":"recipes.html#api-key-authentication"},{"type":"extras","doc":"For more complex authentication flows:\n\n```elixir\ndefmodule MyApp.OAuthResource do\n  use Hermes.Server.Component,\n    type: :resource,\n    uri: \"auth://oauth/status\"\n\n  def read(_params, frame) do\n    case frame.assigns[:oauth_token] do\n      nil ->\n        {:ok, Jason.encode!(%{\n          authenticated: false,\n          login_url: generate_oauth_url(frame.private.session_id)\n        })}\n\n      token ->\n        {:ok, Jason.encode!(%{\n          authenticated: true,\n          user: fetch_user_info(token)\n        })}\n    end\n  end\nend\n```","title":"OAuth Integration - Recipes","ref":"recipes.html#oauth-integration"},{"type":"extras","doc":"Need to work with files? Here's a safe pattern:\n\n```elixir\ndefmodule MyApp.FileManager do\n  use Hermes.Server.Component, type: :tool\n\n  @moduledoc \"Safely read files from allowed directories\"\n\n  schema do\n    field :path, :string, required: true\n  end\n\n  def execute(%{path: path}, frame) do\n    user = frame.assigns.user\n    allowed_dirs = get_allowed_directories(user)\n\n    with :ok <- validate_path_access(path, allowed_dirs),\n         {:ok, content} <- File.read(path) do\n      {:ok, %{\n        path: path,\n        size: byte_size(content),\n        content: content,\n        mime_type: MIME.from_path(path)\n      }}\n    else\n      {:error, :access_denied} ->\n        {:error, \"Access denied to path: #{path}\"}\n\n      {:error, reason} ->\n        {:error, \"Failed to read file: #{inspect(reason)}\"}\n    end\n  end\n\n  defp validate_path_access(path, allowed_dirs) do\n    real_path = Path.expand(path)\n\n    if Enum.any?(allowed_dirs, &String.starts_with?(real_path, &1)) do\n      :ok\n    else\n      {:error, :access_denied}\n    end\n  end\nend\n```","title":"File Operations - Recipes","ref":"recipes.html#file-operations"},{"type":"extras","doc":"How do you expose database queries safely?\n\n```elixir\ndefmodule MyApp.QueryBuilder do\n  use Hermes.Server.Component, type: :tool\n\n  @moduledoc \"Build and execute safe database queries\"\n\n  schema do\n    field :table, :string, required: true, values: [\"users\", \"products\", \"orders\"]\n    field :filters, :map\n    field :limit, :integer, default: 100, max: 1000\n    field :order_by, :string\n  end\n\n  def execute(params, frame) do\n    query =\n      build_base_query(params.table)\n      |> apply_filters(params[:filters])\n      |> apply_order(params[:order_by])\n      |> apply_limit(params.limit)\n      |> apply_user_scope(frame.assigns.user)\n\n    case MyApp.Repo.all(query) do\n      results when is_list(results) ->\n        {:ok, Enum.map(results, &sanitize_result/1)}\n\n      error ->\n        {:error, \"Query failed: #{inspect(error)}\"}\n    end\n  end\n\n  defp build_base_query(\"users\"), do: from(u in User)\n  defp build_base_query(\"products\"), do: from(p in Product)\n  defp build_base_query(\"orders\"), do: from(o in Order)\n\n  defp apply_filters(query, nil), do: query\n  defp apply_filters(query, filters) do\n    Enum.reduce(filters, query, fn {field, value}, q ->\n      where(q, [r], field(r, ^String.to_atom(field)) == ^value)\n    end)\n  end\n\n  defp sanitize_result(record) do\n    # Remove sensitive fields\n    record\n    |> Map.from_struct()\n    |> Map.drop([:password_hash, :api_key, :secret_token])\n  end\nend\n```","title":"Database Operations - Recipes","ref":"recipes.html#database-operations"},{"type":"extras","doc":"What about operations that take time? Let's handle them gracefully:\n\n```elixir\ndefmodule MyApp.ReportGenerator do\n  use Hermes.Server.Component, type: :tool\n\n  @moduledoc \"Generate complex reports\"\n\n  schema do\n    field :report_type, :string, required: true\n    field :date_range, :map\n  end\n\n  def execute(params, frame) do\n    # Start async task\n    task_id = UUID.uuid4()\n\n    Task.start(fn ->\n      result = generate_report(params)\n      ReportStore.save(task_id, result)\n    end)\n\n    # Return immediately with task ID\n    {:ok, %{\n      task_id: task_id,\n      status: \"processing\",\n      check_status_with: \"report_status\"\n    }}\n  end\nend\n\ndefmodule MyApp.ReportStatus do\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :task_id, :string, required: true\n  end\n\n  def execute(%{task_id: task_id}, _frame) do\n    case ReportStore.get(task_id) do\n      nil ->\n        {:ok, %{status: \"processing\"}}\n\n      {:completed, result} ->\n        {:ok, %{status: \"completed\", result: result}}\n\n      {:error, reason} ->\n        {:ok, %{status: \"failed\", error: reason}}\n    end\n  end\nend\n```","title":"Long-Running Operations - Recipes","ref":"recipes.html#long-running-operations"},{"type":"extras","doc":"Need to push updates to clients? Here's how:\n\n```elixir\ndefmodule MyApp.LiveDataServer do\n  use Hermes.Server,\n    name: \"live-data\",\n    version: \"1.0.0\",\n    capabilities: [:tools]\n\n  def init(arg, frame) do\n    # Subscribe to Phoenix PubSub\n    Phoenix.PubSub.subscribe(MyApp.PubSub, \"data_updates\")\n    {:ok, frame}\n  end\n\n  def handle_info({:data_update, data}, frame) do\n    # Send notification to client\n    notification = %{\n      method: \"notifications/resources/list_changed\",\n      params: %{}\n    }\n\n    send_notification(notification)\n    {:noreply, frame}\n  end\nend\n```","title":"Real-time Updates - Recipes","ref":"recipes.html#real-time-updates"},{"type":"extras","doc":"How do you test MCP components effectively?\n\n```elixir\ndefmodule MyApp.ComponentTest do\n  use ExUnit.Case, async: true\n\n  describe \"complex tool validation\" do\n    test \"validates required fields\" do\n      tool = MyApp.ComplexTool\n\n      # Test schema validation\n      assert {:error, errors} =\n        Hermes.Server.Component.validate_params(tool, %{})\n\n      assert errors[:required_field] == [\"is required\"]\n    end\n\n    test \"executes with valid params\" do\n      params = %{required_field: \"value\", optional_field: 42}\n      frame = %Hermes.Server.Frame{assigns: %{user: %{id: 1}}}\n\n      assert {:ok, result} = MyApp.ComplexTool.execute(params, frame)\n      assert result.processed == true\n    end\n  end\nend\n```","title":"Testing Patterns - Recipes","ref":"recipes.html#testing-patterns"},{"type":"extras","doc":"How do you handle and recover from errors gracefully?\n\n```elixir\ndefmodule MyApp.ResilientTool do\n  use Hermes.Server.Component, type: :tool\n\n  def execute(params, frame) do\n    with {:ok, data} <- fetch_external_data(params),\n         {:ok, processed} <- process_data(data),\n         {:ok, stored} <- store_results(processed) do\n      {:ok, format_success(stored)}\n    else\n      {:error, :external_service_down} ->\n        # Fallback to cache\n        case get_cached_data(params) do\n          {:ok, cached} ->\n            {:ok, %{data: cached, source: \"cache\", warning: \"Using cached data\"}}\n          :error ->\n            {:error, \"Service unavailable and no cached data found\"}\n        end\n\n      {:error, :rate_limited} ->\n        {:error, \"Rate limited. Please try again in a few minutes.\"}\n\n      {:error, reason} ->\n        Logger.error(\"Tool execution failed: #{inspect(reason)}\")\n        {:error, \"An unexpected error occurred\"}\n    end\n  end\nend\n```","title":"Error Recovery - Recipes","ref":"recipes.html#error-recovery"},{"type":"extras","doc":"Need to handle high-throughput scenarios?\n\n```elixir\ndefmodule MyApp.BatchProcessor do\n  use Hermes.Server.Component, type: :tool\n\n  schema do\n    field :items, {:array, :map}, required: true, max_items: 1000\n  end\n\n  def execute(%{items: items}, frame) do\n    # Process in parallel with flow control\n    results =\n      items\n      |> Task.async_stream(\n        &process_item/1,\n        max_concurrency: System.schedulers_online() * 2,\n        timeout: 5_000,\n        on_timeout: :kill_task\n      )\n      |> Enum.map(fn\n        {:ok, result} -> result\n        {:exit, :timeout} -> {:error, \"Processing timeout\"}\n      end)\n\n    successful = Enum.filter(results, &match?({:ok, _}, &1))\n    failed = Enum.filter(results, &match?({:error, _}, &1))\n\n    {:ok, %{\n      processed: length(successful),\n      failed: length(failed),\n      results: successful\n    }}\n  end\nend\n```","title":"Performance Optimization - Recipes","ref":"recipes.html#performance-optimization"},{"type":"extras","doc":"How do you track what's happening in your MCP system? Let's explore logging patterns:","title":"Logging & Monitoring - Recipes","ref":"recipes.html#logging-monitoring"},{"type":"extras","doc":"Control log verbosity from servers you connect to:\n\n```elixir\n# Tell the server to only send warnings and above\nMyApp.Client.set_log_level(\"warning\")\n\n# Register a handler for incoming logs\nMyApp.Client.register_log_callback(fn level, data, logger ->\n  # Forward to your monitoring service\n  MyApp.Telemetry.log_event(level, data, source: logger)\n\n  # Or filter specific loggers\n  if logger == \"database\" do\n    Logger.warning(\"Database: #{data}\")\n  end\nend)\n```","title":"MCP Protocol Logging - Recipes","ref":"recipes.html#mcp-protocol-logging"},{"type":"extras","doc":"Send structured logs to connected clients:\n\n```elixir\ndefmodule MyApp.LoggingServer do\n  use Hermes.Server,\n    name: \"logging-demo\",\n    version: \"1.0.0\",\n    capabilities: [:logging]  # Advertise logging support\n\n  def handle_request(%{\"method\" => \"tools/call\"} = request, frame) do\n    # Send log notifications to client\n    send_log_message(self(), \"info\", \"Processing tool request\", \"request_handler\")\n\n    # Do the work...\n    result = process_request(request)\n\n    send_log_message(self(), \"debug\", \"Request completed\", \"request_handler\")\n    {:reply, result, frame}\n  end\nend\n```","title":"Server-Side Logging - Recipes","ref":"recipes.html#server-side-logging"},{"type":"extras","doc":"Control Hermes's internal logging verbosity:\n\n```elixir\n# In config/config.exs\nconfig :hermes_mcp, :logging,\n  client_events: :info,      # Client lifecycle events\n  server_events: :info,      # Server request handling\n  transport_events: :warning, # Connection issues\n  protocol_messages: :debug   # Raw message exchanges\n\n# Or disable completely for production\nconfig :hermes_mcp, log: false\n```","title":"Configuring Library Logging - Recipes","ref":"recipes.html#configuring-library-logging"},{"type":"extras","doc":"These recipes come from real-world usage. What challenges are you facing?\n\n- Complex authentication flows?\n- Multi-step workflows?\n- Integration with existing systems?\n- Performance at scale?\n- Logging and observability?\n\nEach pattern can be adapted to your specific needs. The key insight? MCP handles the protocol complexity while you focus on your domain logic.\n\nReady to implement one of these patterns?","title":"What Pattern Do You Need? - Recipes","ref":"recipes.html#what-pattern-do-you-need"},{"type":"extras","doc":"# API Reference\n\nA quick reference for the most commonly used functions. Looking for more detailed examples? Check out our guides.","title":"API Reference","ref":"reference.html"},{"type":"extras","doc":"","title":"Client API - API Reference","ref":"reference.html#client-api"},{"type":"extras","doc":"```elixir\nuse Hermes.Client, options\n```\n\n**Required Options:**\n\n- `name` - Your client name (string)\n- `version` - Your client version (string)\n- `protocol_version` - MCP protocol version (string)\n- `capabilities` - List of capabilities (atoms or tuples)","title":"Module Definition - API Reference","ref":"reference.html#module-definition"},{"type":"extras","doc":"```elixir\n{MyApp.Client, transport: transport_config}\n```\n\n**Transport Options:**\n\n- `{:stdio, command: \"cmd\", args: [\"arg1\", \"arg2\"]}`\n- `{:streamable_http, url: \"http://localhost:8000/mcp\"}`\n- `{:websocket, url: \"ws://localhost:8000/ws\"}`\n- `{:sse, base_url: \"http://localhost:8000\"}`","title":"Starting a Client - API Reference","ref":"reference.html#starting-a-client"},{"type":"extras","doc":"All functions accept an optional process name as the first argument:\n\n```elixir\n# Default process\nMyApp.Client.ping()\n\n# Named process\nMyApp.Client.ping(:my_client)\n```\n\n**Connection Management:**\n\n- `ping/0,1` - Check if server is responsive\n- `close/0,1` - Close the connection gracefully\n\n**Discovery:**\n\n- `get_server_info/0,1` - Get server information\n- `get_server_capabilities/0,1` - Get server capabilities\n\n**Tools:**\n\n- `list_tools/0,1` - List available tools\n- `call_tool/2,3,4` - Call a tool with arguments\n\n**Resources:**\n\n- `list_resources/0,1,2` - List available resources (supports pagination)\n- `read_resource/1,2` - Read a specific resource\n\n**Prompts:**\n\n- `list_prompts/0,1,2` - List available prompts (supports pagination)\n- `get_prompt/2,3` - Get a prompt with arguments\n\n**Autocompletion:**\n\n- `complete/2,3` - Get completion suggestions","title":"Client Functions - API Reference","ref":"reference.html#client-functions"},{"type":"extras","doc":"Functions that make requests accept options:\n\n- `timeout: milliseconds` - Request timeout (default: 30_000)","title":"Options - API Reference","ref":"reference.html#options"},{"type":"extras","doc":"","title":"Server API - API Reference","ref":"reference.html#server-api"},{"type":"extras","doc":"```elixir\nuse Hermes.Server, options\n```\n\n**Required Options:**\n\n- `name` - Your server name (string)\n- `version` - Your server version (string)\n- `capabilities` - List of capabilities to expose","title":"Module Definition - API Reference","ref":"reference.html#module-definition-1"},{"type":"extras","doc":"```elixir\n{MyApp.Server, transport: transport_config}\n```\n\n**Transport Options:**\n\n- `:stdio` - Standard input/output\n- `{:streamable_http, port: 8080}` - HTTP server\n- `:none` - No transport (for embedding)","title":"Starting a Server - API Reference","ref":"reference.html#starting-a-server"},{"type":"extras","doc":"```elixir\n@behaviour Hermes.Server.Behaviour\n\n# Optional initialization\ndef init(arg, frame) do\n  {:ok, frame}\nend\n\n# Handle incoming requests (optional)\ndef handle_request(request, frame) do\n  {:reply, result, frame}\nend\n\n# Handle notifications (optional)\ndef handle_notification(notification, frame) do\n  {:noreply, frame}\nend\n```","title":"Server Callbacks - API Reference","ref":"reference.html#server-callbacks"},{"type":"extras","doc":"#### Tools\n\n```elixir\nuse Hermes.Server.Component, type: :tool\n\n# Schema definition\nschema do\n  field :name, :string, required: true\n  field :age, :integer, min: 0\nend\n\n# Execution callback\ndef execute(params, frame) do\n  {:ok, result}  # or {:error, reason}\nend\n```\n\n#### Resources\n\n```elixir\nuse Hermes.Server.Component,\n  type: :resource,\n  uri: \"resource://type/name\"\n\n# Read callback\ndef read(params, frame) do\n  {:ok, content}  # or {:error, reason}\nend\n```\n\n#### Prompts\n\n```elixir\nuse Hermes.Server.Component, type: :prompt\n\n# Schema for arguments\nschema do\n  field :context, :string\nend\n\n# Get messages callback\ndef get_messages(params, frame) do\n  {:ok, [%{role: \"user\", content: \"...\"}]}\nend\n```","title":"Component Definition - API Reference","ref":"reference.html#component-definition"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Server do\n  use Hermes.Server, ...\n\n  # Register components\n  component MyApp.MyTool\n  component MyApp.MyResource\n  component MyApp.MyPrompt\nend\n```","title":"Component Registration - API Reference","ref":"reference.html#component-registration"},{"type":"extras","doc":"Available field types and validations:\n\n```elixir\nschema do\n  field :string_field, :string,\n    required: true,\n    min_length: 1,\n    max_length: 100,\n    format: ~r/pattern/\n\n  field :number_field, :number,\n    min: 0,\n    max: 100\n\n  field :integer_field, :integer,\n    min: 0\n\n  field :boolean_field, :boolean,\n    default: false\n\n  field :enum_field, :string,\n    values: [\"option1\", \"option2\"]\n\n  field :array_field, {:array, :string},\n    min_items: 1,\n    max_items: 10\n\n  field :map_field, :map\nend\n```","title":"Schema DSL - API Reference","ref":"reference.html#schema-dsl"},{"type":"extras","doc":"","title":"Return Values - API Reference","ref":"reference.html#return-values"},{"type":"extras","doc":"Most client functions return:\n\n- `{:ok, %{result: data}}` - Successful response\n- `{:ok, %{is_error: true, result: error}}` - Tool-level error\n- `{:error, reason}` - Protocol or connection error","title":"Client Returns - API Reference","ref":"reference.html#client-returns"},{"type":"extras","doc":"Component callbacks should return:\n\n- `{:ok, result}` - Success with result\n- `{:error, message}` - Error with message\n\nServer callbacks return:\n\n- `{:reply, result, frame}` - Reply with result\n- `{:noreply, frame}` - No reply\n- `{:stop, reason, frame}` - Stop the server","title":"Server Returns - API Reference","ref":"reference.html#server-returns"},{"type":"extras","doc":"Errors are automatically formatted according to MCP protocol. You can return:\n\n- String error messages\n- Error tuples\n- Exceptions (will be caught and formatted)","title":"Error Handling - API Reference","ref":"reference.html#error-handling"},{"type":"extras","doc":"**Interactive Testing:**\n\n- `mix hermes.stdio.interactive Module` - Test with STDIO\n- `mix hermes.streamable_http.interactive Module` - Test with HTTP\n\n**Development:**\n\n- `mix compile --force` - Recompile all components\n- `mix test` - Run tests","title":"Useful Mix Tasks - API Reference","ref":"reference.html#useful-mix-tasks"},{"type":"extras","doc":"This reference covers the essential API. For detailed examples and patterns:\n\n- [Building a Client](building-a-client.md)\n- [Building a Server](building-a-server.md)\n- [Recipes](recipes.md)\n\nRemember: the protocol complexity is handled for you. Focus on what your application does best.","title":"Need More? - API Reference","ref":"reference.html#need-more"},{"type":"extras","doc":"# Changelog\n\nAll notable changes to this project are documented in this file.\n\n## [0.11.3](https://github.com/cloudwalk/hermes-mcp/compare/v0.11.2...v0.11.3) (2025-07-02)","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"* correctly parse dates when default values are passed ([58f6368](https://github.com/cloudwalk/hermes-mcp/commit/58f63686574e230269f847ee793d9561976653bb))\n* include frame helpers on module-based component ([#163](https://github.com/cloudwalk/hermes-mcp/issues/163)) ([15ba2c7](https://github.com/cloudwalk/hermes-mcp/commit/15ba2c7fbdd0a776eabe028aef0350f4f52a43a8))\n* server can now send notifications correctly ([#166](https://github.com/cloudwalk/hermes-mcp/issues/166)) ([33f32de](https://github.com/cloudwalk/hermes-mcp/commit/33f32deccd42dc4591832636b2b3fad56ce40661))","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* update documentation, simplify, more storytelling ([#168](https://github.com/cloudwalk/hermes-mcp/issues/168)) ([ccdddc1](https://github.com/cloudwalk/hermes-mcp/commit/ccdddc1478c09e2f3440a6f1343949aeca854a1d))\n\n## [0.11.2](https://github.com/cloudwalk/hermes-mcp/compare/v0.11.1...v0.11.2) (2025-06-30)","title":"Miscellaneous Chores - Changelog","ref":"changelog.html#miscellaneous-chores"},{"type":"extras","doc":"- correctly parse peri numeric contrainsts to json-schema ([#160](https://github.com/cloudwalk/hermes-mcp/issues/160)) ([808c2c0](https://github.com/cloudwalk/hermes-mcp/commit/808c2c09e490bf9a866d5575abbf08d355c8324b))\n- interactive http tasks should accept custom headers ([#159](https://github.com/cloudwalk/hermes-mcp/issues/159)) ([c2fe91e](https://github.com/cloudwalk/hermes-mcp/commit/c2fe91eff5a701e42a636ee1a291ea51a93f7983))\n\n## [0.11.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.11.0...v0.11.1) (2025-06-30)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-1"},{"type":"extras","doc":"- loggin should respect the logger config ([#157](https://github.com/cloudwalk/hermes-mcp/issues/157)) ([0fbf6a6](https://github.com/cloudwalk/hermes-mcp/commit/0fbf6a652ed9dd5da64716d47ece675192442ea0))\n\n## [0.11.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.5...v0.11.0) (2025-06-30)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-2"},{"type":"extras","doc":"- runtime server components, simplified api ([#153](https://github.com/cloudwalk/hermes-mcp/issues/153)) ([8af35d6](https://github.com/cloudwalk/hermes-mcp/commit/8af35d67cd15d125e40ab9b115ff6900c3487ea5))","title":"Features - Changelog","ref":"changelog.html#features"},{"type":"extras","doc":"- allow users to control external process messages ([#149](https://github.com/cloudwalk/hermes-mcp/issues/149)) ([8fef4eb](https://github.com/cloudwalk/hermes-mcp/commit/8fef4eb753f38a325a1d3a94c310e5fd1c478ede))\n- correctly encode prompt/resource ([#155](https://github.com/cloudwalk/hermes-mcp/issues/155)) ([4249b13](https://github.com/cloudwalk/hermes-mcp/commit/4249b137e43587862de2e59acba4660ac785702a))\n- more genserver mcp server callbacks, plug based startup and documentation ([#152](https://github.com/cloudwalk/hermes-mcp/issues/152)) ([9c26b1c](https://github.com/cloudwalk/hermes-mcp/commit/9c26b1ce4d033e3c69bc872a5ed01a037ec68f59))\n- server behaviour with optional callbacks ([#151](https://github.com/cloudwalk/hermes-mcp/issues/151)) ([91aa191](https://github.com/cloudwalk/hermes-mcp/commit/91aa1916f28da5972f6b100861a2547697c1ddb7))","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-3"},{"type":"extras","doc":"- improve runtime components schema def ([#154](https://github.com/cloudwalk/hermes-mcp/issues/154)) ([96ff2a9](https://github.com/cloudwalk/hermes-mcp/commit/96ff2a97be02f0f35ee4a288838a1dbf518bbac3))\n\n## [0.10.5](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.4...v0.10.5) (2025-06-25)","title":"Code Refactoring - Changelog","ref":"changelog.html#code-refactoring"},{"type":"extras","doc":"- normalize transport api ([#146](https://github.com/cloudwalk/hermes-mcp/issues/146)) ([8a30a34](https://github.com/cloudwalk/hermes-mcp/commit/8a30a34c944ca4d85de622f9063a558bd495c6fc)), closes [#145](https://github.com/cloudwalk/hermes-mcp/issues/145)\n\n## [0.10.4](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.3...v0.10.4) (2025-06-24)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-4"},{"type":"extras","doc":"- server session expiration on idle (configurable) ([#143](https://github.com/cloudwalk/hermes-mcp/issues/143)) ([d9f7164](https://github.com/cloudwalk/hermes-mcp/commit/d9f7164028b2b6e8ff3d697f888adf655110bc1f))\n\n## [0.10.3](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.2...v0.10.3) (2025-06-23)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-5"},{"type":"extras","doc":"- include formatter on hex release ([#139](https://github.com/cloudwalk/hermes-mcp/issues/139)) ([d91b244](https://github.com/cloudwalk/hermes-mcp/commit/d91b244aeed4211a6653a82705b03da2247db9a3))\n\n## [0.10.2](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.1...v0.10.2) (2025-06-23)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-6"},{"type":"extras","doc":"- default implementation for server handle_notification ([#135](https://github.com/cloudwalk/hermes-mcp/issues/135)) ([c958041](https://github.com/cloudwalk/hermes-mcp/commit/c9580410162c31a62190631f6702024ea3458beb))","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-7"},{"type":"extras","doc":"- cleaner peri integration ([#137](https://github.com/cloudwalk/hermes-mcp/issues/137)) ([43226cc](https://github.com/cloudwalk/hermes-mcp/commit/43226cc9fb2f49edfbf8685fe181eb3093304308)), closes [#123](https://github.com/cloudwalk/hermes-mcp/issues/123)\n\n## [0.10.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.10.0...v0.10.1) (2025-06-21)","title":"Code Refactoring - Changelog","ref":"changelog.html#code-refactoring-1"},{"type":"extras","doc":"- client should send both sse/json headers on POST requests ([#134](https://github.com/cloudwalk/hermes-mcp/issues/134)) ([e906b7f](https://github.com/cloudwalk/hermes-mcp/commit/e906b7f02bf390faecc2b6bd39aab05ef9c500b1))\n- correctly allows macro-based/callback-based server implementations ([#131](https://github.com/cloudwalk/hermes-mcp/issues/131)) ([d7bfc75](https://github.com/cloudwalk/hermes-mcp/commit/d7bfc7541a8c381573f1a20ebc37d4a7dbaaa139))\n- remove last uses of hard-coded Hermes.Server.Registry ([cc0ffd9](https://github.com/cloudwalk/hermes-mcp/commit/cc0ffd95fce771a9c861c6859124dcedb6ceb88e))\n\n## [0.10.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.9.1...v0.10.0) (2025-06-18)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-8"},{"type":"extras","doc":"- batch operations on server-side ([#125](https://github.com/cloudwalk/hermes-mcp/issues/125)) ([28eea7c](https://github.com/cloudwalk/hermes-mcp/commit/28eea7cd15f72c4effccc8475e6b301b2cd9745c))\n- missing notifications handlers ([#129](https://github.com/cloudwalk/hermes-mcp/issues/129)) ([34d5934](https://github.com/cloudwalk/hermes-mcp/commit/34d593499fdd846f430b93b4c52ca986f345646d))\n- support batch operations on client side ([#101](https://github.com/cloudwalk/hermes-mcp/issues/101)) ([fadf28d](https://github.com/cloudwalk/hermes-mcp/commit/fadf28d80068f3f1e77835fd46a276338048f0bc))\n- tools annotations ([#127](https://github.com/cloudwalk/hermes-mcp/issues/127)) ([c83e8f1](https://github.com/cloudwalk/hermes-mcp/commit/c83e8f1b0e721b1a03960ac67cdd0774337675dc))","title":"Features - Changelog","ref":"changelog.html#features-1"},{"type":"extras","doc":"- release please correct version on readme ([#128](https://github.com/cloudwalk/hermes-mcp/issues/128)) ([d0125c6](https://github.com/cloudwalk/hermes-mcp/commit/d0125c664b5190b560bb988ff01d17cbdba814bd))\n- update peri ([#126](https://github.com/cloudwalk/hermes-mcp/issues/126)) ([7292615](https://github.com/cloudwalk/hermes-mcp/commit/7292615cf57da0185006df2f6221f8ab93aacd30))\n\n## [0.9.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.9.0...v0.9.1) (2025-06-13)","title":"Miscellaneous Chores - Changelog","ref":"changelog.html#miscellaneous-chores-1"},{"type":"extras","doc":"- allow enum specific type on json-schema ([#121](https://github.com/cloudwalk/hermes-mcp/issues/121)) ([23c9ce2](https://github.com/cloudwalk/hermes-mcp/commit/23c9ce2081ed1099ce1f3afbd9318c8a02480039)), closes [#114](https://github.com/cloudwalk/hermes-mcp/issues/114)\n- correctly escape quoted expressions ([#119](https://github.com/cloudwalk/hermes-mcp/issues/119)) ([0c469c5](https://github.com/cloudwalk/hermes-mcp/commit/0c469c552d8d5fd2498706b4b1f41100e8561e2f)), closes [#118](https://github.com/cloudwalk/hermes-mcp/issues/118)\n\n## [0.9.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.2...v0.9.0) (2025-06-12)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-9"},{"type":"extras","doc":"- add independently configurable logs ([#113](https://github.com/cloudwalk/hermes-mcp/issues/113)) ([bb0be27](https://github.com/cloudwalk/hermes-mcp/commit/bb0be2716ac41bf43814a16497b209fa55cc811b))","title":"Features - Changelog","ref":"changelog.html#features-2"},{"type":"extras","doc":"- allow registering a name for the client supervisor ([#117](https://github.com/cloudwalk/hermes-mcp/issues/117)) ([d356511](https://github.com/cloudwalk/hermes-mcp/commit/d356511e2fbcf47b6a85ed733671e96d800ac693))\n\n## [0.8.2](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.1...v0.8.2) (2025-06-11)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-10"},{"type":"extras","doc":"- higher-level client implementation ([#111](https://github.com/cloudwalk/hermes-mcp/issues/111)) ([5de2162](https://github.com/cloudwalk/hermes-mcp/commit/5de2162b166e390b4dacfd7b9960fab59c81b4d7))\n\n## [0.8.1](https://github.com/cloudwalk/hermes-mcp/compare/v0.8.0...v0.8.1) (2025-06-10)","title":"Code Refactoring - Changelog","ref":"changelog.html#code-refactoring-2"},{"type":"extras","doc":"- hermes should respect mix releases startup ([#109](https://github.com/cloudwalk/hermes-mcp/issues/109)) ([f42d476](https://github.com/cloudwalk/hermes-mcp/commit/f42d476e1dc05c57479170bd58aaca9028ef1e66))\n\n## [0.8.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.7.0...v0.8.0) (2025-06-10)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-11"},{"type":"extras","doc":"- inject user and transport data on mcp server frame ([#106](https://github.com/cloudwalk/hermes-mcp/issues/106)) ([feb2ce3](https://github.com/cloudwalk/hermes-mcp/commit/feb2ce308e9fd0cde4118b294dd47ce64d8db18f))\n- legacy sse server transport ([#102](https://github.com/cloudwalk/hermes-mcp/issues/102)) ([4a71088](https://github.com/cloudwalk/hermes-mcp/commit/4a71088713071a726bde03bf1385c3c794d2134b))","title":"Features - Changelog","ref":"changelog.html#features-3"},{"type":"extras","doc":"- allow empty capabilities on incoming JSON-RPC messages ([#105](https://github.com/cloudwalk/hermes-mcp/issues/105)) ([f0ad4cf](https://github.com/cloudwalk/hermes-mcp/commit/f0ad4cf1a5a85cc8a56baed875d2d2d200bb5860)), closes [#96](https://github.com/cloudwalk/hermes-mcp/issues/96)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-12"},{"type":"extras","doc":"- release please should include all files ([#108](https://github.com/cloudwalk/hermes-mcp/issues/108)) ([d0a25b9](https://github.com/cloudwalk/hermes-mcp/commit/d0a25b968c83ae1023ffffc8ee07b5b490122c03))\n\n## [0.7.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.6.0...v0.7.0) (2025-06-09)","title":"Miscellaneous Chores - Changelog","ref":"changelog.html#miscellaneous-chores-2"},{"type":"extras","doc":"- allow json schema fields on tools/prompts definition ([#99](https://github.com/cloudwalk/hermes-mcp/issues/99)) ([0345f12](https://github.com/cloudwalk/hermes-mcp/commit/0345f122484a0169645c5da07e50c2d64fd6c5f5))\n\n## [0.6.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.5.0...v0.6.0) (2025-06-09)","title":"Features - Changelog","ref":"changelog.html#features-4"},{"type":"extras","doc":"- allow customize server registry impl ([#94](https://github.com/cloudwalk/hermes-mcp/issues/94)) ([f3ac087](https://github.com/cloudwalk/hermes-mcp/commit/f3ac08749a7c361466a7a619f9782e8d8706a7b6))\n- mcp high level server components definition ([#91](https://github.com/cloudwalk/hermes-mcp/issues/91)) ([007f41d](https://github.com/cloudwalk/hermes-mcp/commit/007f41d33874fd9f1b5e340ecbe16317dc3576b7))\n- mcp server handlers refactored ([#92](https://github.com/cloudwalk/hermes-mcp/issues/92)) ([e213e04](https://github.com/cloudwalk/hermes-mcp/commit/e213e046b1360b24ff9e42835cdf80f5fe2ae4fa))","title":"Features - Changelog","ref":"changelog.html#features-5"},{"type":"extras","doc":"- correctly handle mcp requests on phoenix apps ([#88](https://github.com/cloudwalk/hermes-mcp/issues/88)) ([09f4235](https://github.com/cloudwalk/hermes-mcp/commit/09f42359f0daac694013f0be4f6a74de2be7f4ff)), closes [#86](https://github.com/cloudwalk/hermes-mcp/issues/86)","title":"Bug Fixes - Changelog","ref":"changelog.html#bug-fixes-13"},{"type":"extras","doc":"- upcate automatic version ([#98](https://github.com/cloudwalk/hermes-mcp/issues/98)) ([0c08233](https://github.com/cloudwalk/hermes-mcp/commit/0c08233371338af24ea66047b4e1a8e9fa5cb055))","title":"Miscellaneous Chores - Changelog","ref":"changelog.html#miscellaneous-chores-3"},{"type":"extras","doc":"- tests ([#93](https://github.com/cloudwalk/hermes-mcp/issues/93)) ([ca31feb](https://github.com/cloudwalk/hermes-mcp/commit/ca31febee7aec1d45dcb32398b33228d1399ae39))\n\n## [0.5.0](https://github.com/cloudwalk/hermes-mcp/compare/v0.4.1...v0.5.0) (2025-06-05)","title":"Code Refactoring - Changelog","ref":"changelog.html#code-refactoring-3"},{"type":"extras","doc":"- client support new mcp spec ([#83](https://github.com/cloudwalk/hermes-mcp/issues/83)) ([73d14f7](https://github.com/cloudwalk/hermes-mcp/commit/73d14f77522cef0f7212230c05cdac23ee2d93e2))\n- enable log disabling ([#78](https://github.com/cloudwalk/hermes-mcp/issues/78)) ([fa1453f](https://github.com/cloudwalk/hermes-mcp/commit/fa1453fee9b015c0ad7f9ac223749a9c9f1fcf6a))\n- low level genservy mcp server implementation (stdio + stremable http) ([#77](https://github.com/cloudwalk/hermes-mcp/issues/77)) ([e6606b4](https://github.com/cloudwalk/hermes-mcp/commit/e6606b4d66a2d7ddeb6c32e0041c22d4f0036ac5))\n- mvp higher level mcp server definition ([#84](https://github.com/cloudwalk/hermes-mcp/issues/84)) ([a5fec1c](https://github.com/cloudwalk/hermes-mcp/commit/a5fec1c976595c3363d4eec83e0cbc382eac9207))","title":"Features - Changelog","ref":"changelog.html#features-6"},{"type":"extras","doc":"- base mcp server implementation correctly uses streamable http ([#85](https://github.com/cloudwalk/hermes-mcp/issues/85)) ([29060fd](https://github.com/cloudwalk/hermes-mcp/commit/29060fd2d2e383c58c727a9085b30162c6b8179a))\n\n## [0.4.0](https://github.com/cloudwalk/hermes-mcp) - 2025-05-06","title":"Code Refactoring - Changelog","ref":"changelog.html#code-refactoring-4"},{"type":"extras","doc":"- Implemented WebSocket transport (#70)\n- Emit `telemetry` events (#54)\n- Implement client feature `completion` request (#72)\n- Implement client feature roots, server requests (#73)\n\n## [0.3.12](https://github.com/cloudwalk/hermes-mcp) - 2025-04-24","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Correctly handles \"nested\" timeouts (genserver vs MCP) (#71)\n\n## [0.3.11](https://github.com/cloudwalk/hermes-mcp) - 2025-04-17","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Improved core library logging and added verbosity level on interactive/CLI (#68)\n\n## [0.3.10](https://github.com/cloudwalk/hermes-mcp) - 2025-04-17","title":"Added - Changelog","ref":"changelog.html#added-1"},{"type":"extras","doc":"- Handle SSE ping and reconnect events from server (#65)\n\n## [0.3.9](https://github.com/cloudwalk/hermes-mcp) - 2025-04-15","title":"Fixed - Changelog","ref":"changelog.html#fixed-1"},{"type":"extras","doc":"- Improved and simplified SSE endpoint event URI merging (#64)","title":"Fixed - Changelog","ref":"changelog.html#fixed-2"},{"type":"extras","doc":"- Added internal client/transport state inspection on CLI/mix tasks (#61)\n\n## [0.3.8](https://github.com/cloudwalk/hermes-mcp) - 2025-04-10","title":"Added - Changelog","ref":"changelog.html#added-2"},{"type":"extras","doc":"- Created `Operation` struct to standardize client API calls (#56)\n- Fixed ERTS version to avoid release errors","title":"Added - Changelog","ref":"changelog.html#added-3"},{"type":"extras","doc":"- Resolved client timeout confusion by standardizing timeout handling (#42)\n\n## [0.3.7](https://github.com/cloudwalk/hermes-mcp) - 2025-04-01","title":"Fixed - Changelog","ref":"changelog.html#fixed-3"},{"type":"extras","doc":"- Client reinitialization from interactive CLI (#55)\n\n## [0.3.6](https://github.com/cloudwalk/hermes-mcp) - 2025-03-28","title":"Fixed - Changelog","ref":"changelog.html#fixed-4"},{"type":"extras","doc":"- New roadmap and protocol update proposal (#53)\n- Added documentation for the 2025-03-26 protocol update\n\n## [0.3.5](https://github.com/cloudwalk/hermes-mcp) - 2025-03-25","title":"Added - Changelog","ref":"changelog.html#added-4"},{"type":"extras","doc":"- Added Roadmap to README (#47)\n\n## [0.3.4](https://github.com/cloudwalk/hermes-mcp) - 2025-03-20","title":"Documentation - Changelog","ref":"changelog.html#documentation"},{"type":"extras","doc":"- `help` command and flag on the interactive CLI (#37)\n- improve SSE connection status on interactive task/cli (#37)\n\n## [0.3.3](https://github.com/cloudwalk/hermes-mcp) - 2025-03-20","title":"Added - Changelog","ref":"changelog.html#added-5"},{"type":"extras","doc":"- Client request cancellation support (#35)\n- Improved URI path handling for SSE transport (#36)\n- Enhanced interactive mix tasks for testing MCP servers (#34)\n\n## [0.3.2](https://github.com/cloudwalk/hermes-mcp) - 2025-03-19","title":"Added - Changelog","ref":"changelog.html#added-6"},{"type":"extras","doc":"- Ship static binaries to use hermes-mcp as standalone application\n\n## [0.3.1](https://github.com/cloudwalk/hermes-mcp) - 2025-03-19","title":"Added - Changelog","ref":"changelog.html#added-7"},{"type":"extras","doc":"- Ship interactive mix tasks `stdio.interactive` and `sse.interactive` to test MCP servers\n\n## [0.3.0](https://github.com/cloudwalk/hermes-mcp) - 2025-03-18","title":"Added - Changelog","ref":"changelog.html#added-8"},{"type":"extras","doc":"- Structured server-client logging support (#27)\n- Progress notification tracking (#26)\n- MCP domain model implementation (#28)\n- Comprehensive SSE unit tests (#20)\n- Centralized state management (#31)\n- Standardized error response handling (#32)","title":"Added - Changelog","ref":"changelog.html#added-9"},{"type":"extras","doc":"- Improved domain error handling (#33)\n\n## [0.2.3](https://github.com/cloudwalk/hermes-mcp) - 2025-03-12","title":"Fixed - Changelog","ref":"changelog.html#fixed-5"},{"type":"extras","doc":"- Enhanced SSE transport with graceful shutdown capabilities (#25)\n- Improved SSE streaming with automatic reconnection handling (#25)\n\n## [0.2.2](https://github.com/cloudwalk/hermes-mcp) - 2025-03-05","title":"Added - Changelog","ref":"changelog.html#added-10"},{"type":"extras","doc":"- Support for multiple concurrent client <> transport pairs (#24)\n- Improved client resource management\n\n## [0.2.1](https://github.com/cloudwalk/hermes-mcp) - 2025-02-28","title":"Added - Changelog","ref":"changelog.html#added-11"},{"type":"extras","doc":"- Support for custom base and SSE paths in HTTP/SSE client (#19)\n- Enhanced configuration options for SSE endpoints\n\n## [0.2.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-27","title":"Added - Changelog","ref":"changelog.html#added-12"},{"type":"extras","doc":"- Implemented HTTP/SSE transport (#7)\n  - Support for server-sent events communication\n  - HTTP client integration for MCP protocol\n  - Streaming response handling","title":"Added - Changelog","ref":"changelog.html#added-13"},{"type":"extras","doc":"- Extensive guides and documentation improvements\n\n## [0.1.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-26","title":"Documentation - Changelog","ref":"changelog.html#documentation-1"},{"type":"extras","doc":"- Implemented STDIO transport (#1) for MCP communication\n  - Support for bidirectional communication via standard I/O\n  - Automatic process monitoring and recovery\n  - Environment variable handling for cross-platform support\n  - Integration test utilities in Mix tasks\n\n- Created stateful client interface (#6)\n  - Robust GenServer implementation for MCP client\n  - Automatic initialization and protocol handshake\n  - Synchronous-feeling API over asynchronous transport\n  - Support for all MCP operations (ping, resources, prompts, tools)\n  - Proper error handling and logging\n  - Capability negotiation and management\n\n- Developed JSON-RPC message parsing (#5)\n  - Schema-based validation of MCP messages\n  - Support for requests, responses, notifications, and errors\n  - Comprehensive test suite for message handling\n  - Encoding/decoding functions with proper validation\n\n- Established core architecture and client API\n  - MCP protocol implementation following specification\n  - Client struct for maintaining connection state\n  - Request/response correlation with unique IDs\n  - Initial transport abstraction layer","title":"Added - Changelog","ref":"changelog.html#added-14"},{"type":"extras","doc":"- Added detailed RFC document describing the library architecture\n- Enhanced README with project overview and installation instructions","title":"Documentation - Changelog","ref":"changelog.html#documentation-2"},{"type":"extras","doc":"# Contributing to Hermes MCP\n\nThank you for your interest in contributing to Hermes MCP! This document provides guidelines and instructions for contributing to the project.\n\nFirstly, have sure to follow the official MCP (Model Context Protocol) [specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/)!","title":"Contributing to Hermes MCP","ref":"contributing.html"},{"type":"extras","doc":"","title":"Development Setup - Contributing to Hermes MCP","ref":"contributing.html#development-setup"},{"type":"extras","doc":"- Elixir 1.18+\n- Erlang/OTP 26+\n- Python 3.11+ with uv (for echo server)\n- Go 1.21+ (for calculator server)\n- Just command runner","title":"Prerequisites - Contributing to Hermes MCP","ref":"contributing.html#prerequisites"},{"type":"extras","doc":"1. Clone the repository\n\n   ```bash\n   git clone https://github.com/cloudwalk/hermes-mcp.git\n   cd hermes-mcp\n   ```\n\n2. Install dependencies\n\n   ```bash\n   mix setup\n   ```\n\n3. Run tests to ensure everything is working\n   ```bash\n   mix test\n   ```","title":"Getting Started - Contributing to Hermes MCP","ref":"contributing.html#getting-started"},{"type":"extras","doc":"","title":"Development Workflow - Contributing to Hermes MCP","ref":"contributing.html#development-workflow"},{"type":"extras","doc":"For development and testing, you can use the provided MCP server implementations:\n\n```bash\n# Start the Echo server (Python)\n# For now only support stdio transport\njust echo-server\n\n# Start the Calculator server (Go)\n# Supports both stdio and http/sse transport\njust calculator-server sse\n```","title":"Running MCP Servers - Contributing to Hermes MCP","ref":"contributing.html#running-mcp-servers"},{"type":"extras","doc":"Before submitting a pull request, ensure your code passes all quality checks:\n\n```bash\n# Run all code quality checks\nmix lint\n\n# Individual checks\nmix format        # Code formatting\nmix credo         # Linting\nmix dialyzer      # Type checking\n```","title":"Code Quality - Contributing to Hermes MCP","ref":"contributing.html#code-quality"},{"type":"extras","doc":"Write tests for all new features and bug fixes:\n\n```bash\n# Run all tests\nmix test\n```","title":"Testing - Contributing to Hermes MCP","ref":"contributing.html#testing"},{"type":"extras","doc":"1. Create a new branch for your feature or bugfix\n\n   ```bash\n   git checkout -b feature/your-feature-name\n   ```\n\n2. Make your changes and commit them with clear, descriptive messages\n\n3. Push your branch to GitHub\n\n   ```bash\n   git push origin feature/your-feature-name\n   ```\n\n4. Open a pull request against the main branch","title":"Submitting Contributions - Contributing to Hermes MCP","ref":"contributing.html#submitting-contributions"},{"type":"extras","doc":"- Follow the existing code style and conventions\n- Include tests for new functionality\n- Update documentation as needed\n- Keep pull requests focused on a single topic\n- Reference any related issues in your PR description","title":"Pull Request Guidelines - Contributing to Hermes MCP","ref":"contributing.html#pull-request-guidelines"},{"type":"extras","doc":"Update documentation for any user-facing changes:\n\n- Update relevant sections in `pages/` directory\n- Add examples for new features\n- Document any breaking changes","title":"Documentation - Contributing to Hermes MCP","ref":"contributing.html#documentation"},{"type":"extras","doc":"Releases are managed by the maintainers. Version numbers follow [Semantic Versioning](https://semver.org/).","title":"Release Process - Contributing to Hermes MCP","ref":"contributing.html#release-process"},{"type":"extras","doc":"By contributing to Hermes MCP, you agree that your contributions will be licensed under the project's [MIT License](./LICENSE).","title":"License - Contributing to Hermes MCP","ref":"contributing.html#license"},{"type":"extras","doc":"Copyright 2025 zoedsoupe  \n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","title":"LICENSE","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}